#import "Fiend.h"

RCS_ID ("$Id$");

#import "Brain.h"

#import "ActiveApplicationsController.h"
#import "InfoPanelController.h"
#import "FIDock.h"
#import "FIInspectorManager.h"
#import "FIWindowController.h"
#import "FIFindController.h"

@interface NSWindow (Other)
- (void) undo:sender;
- (void) redo:sender;
@end


@implementation Brain

+ (void) initialize
{
    static BOOL initialized = NO;

    if (initialized == YES)
        return;

    initialized = YES;

    [[NSUserDefaults standardUserDefaults] registerDefaults:[NSDictionary dictionaryWithObject:@"YES" forKey:DK_IsDockVisible]];
}

//----------------------------------------------------------------------

- init
{
    if ([super init] == nil)
        return nil;

    windowController = nil;
    document = nil;
    autoSaveTimer = nil;

    return self;
}

//----------------------------------------------------------------------

- (void) dealloc
{
    [self stopAutoSaveTimer];
    SNRelease (document);
    SNRelease (windowController);

    [super dealloc];
}

//----------------------------------------------------------------------

- (void) applicationDidFinishLaunching:(NSNotification *)aNotification
{
    OFUserDefaults *defaults;
    NSWorkspace *sharedWorkspace;
    NSNotificationCenter *workspaceCenter;

    NSFileManager *fileManager;
    NSString *dockPath;
    NSArray *windowControllers;
    double initialFileIconLoadDelay;

    defaults = [OFUserDefaults sharedUserDefaults];

    // Under RDR1 and RDR2, when more than one object listens to the shared workspace,
    // none of them get the notification.  So just have one object listen to the shared
    // workspace, and repost the notifications on another notification center.

    sharedWorkspace = [NSWorkspace sharedWorkspace];
    workspaceCenter = [sharedWorkspace notificationCenter];

    [workspaceCenter addObserver:self
                     selector:@selector (forwardWorkspaceNotification:)
                     name:nil
                     object:sharedWorkspace];

    // The dock is saved in ~/Library/Fiend, so make sure it exists.
    fileManager = [NSFileManager defaultManager];
    [fileManager createDirectoryAtPath:[@"~/Library" stringByStandardizingPath] attributes:nil];
    [fileManager createDirectoryAtPath:[@"~/Library/Fiend" stringByStandardizingPath] attributes:nil];

    if ([defaults boolForKey:DK_ShowInspectorPanel] == YES)
        [self showInspectorPanel:self];

    [self _moveDocksToNewFilenames];

    dockPath = [self dockPath];
    if ([fileManager fileExistsAtPath:dockPath] == NO)
    {
        NSDictionary *emptyDictionary;
        emptyDictionary = [NSDictionary dictionary];
        [emptyDictionary writeToFile:dockPath atomically:YES];
    }
    
    document = [[NSDocumentController sharedDocumentController] openDocumentWithContentsOfFile:dockPath display:YES];

    windowControllers = [document windowControllers];
    OBPRECONDITION ([windowControllers count] == 1);
    windowController = [[windowControllers objectAtIndex:0] retain];
    [[FIInspectorManager sharedInspectorManager] setUndoManager:[windowController undoManager]];
    [windowController performSelector:@selector (performAutoLaunch:) withObject:self afterDelay:0];
    initialFileIconLoadDelay = [defaults floatForKey:DK_InitialFileIconLoadDelay];
    [windowController performSelector:@selector (delayedLoadFileIcons:) withObject:self afterDelay:initialFileIconLoadDelay];
    [self startAutoSaveTimer];

    if ([defaults boolForKey:DK_MakeActive] == YES)
        [NSApp activateIgnoringOtherApps:YES];
}

//----------------------------------------------------------------------

- (void) applicationWillTerminate:(NSNotification *)aNotification
{
    [self stopAutoSaveTimer];
    if ([document isDocumentEdited] == YES)
        [document saveDocument:self];
}

//----------------------------------------------------------------------
#if 0
- (BOOL) applicationShouldTerminate:(NSApplication *)sender
{
    // This doesn't get called before the stupid alert panel!
    [self stopAutoSaveTimer];
    if ([document isDocumentEdited] == YES)
        [document saveDocument:self];
    return YES;
}
#endif
//----------------------------------------------------------------------

- (void) workspaceNotification:(NSNotification *)aNotification
{
    //NSLog (@"notification: %@", aNotification);
    [[NSNotificationCenter fixedWorkspaceCenter] postNotification:aNotification];
}

//----------------------------------------------------------------------

- (void) showInfoPanel:sender
{
    [[InfoPanelController sharedInfoPanelController] showWindow:sender];
}

//----------------------------------------------------------------------

- (void) showPreferencePanel:sender
{
    [[OAPreferenceController sharedPreferenceController] showPreferencesPanel:sender];
}

//----------------------------------------------------------------------

- (void) showActiveApplications:sender
{
    [[ActiveApplicationsController sharedActiveApplicationsController] showWindow:sender];
}

//----------------------------------------------------------------------

- (void) showInspectorPanel:sender
{
    [[FIInspectorManager sharedInspectorManager] showWindow:self];
}

//----------------------------------------------------------------------

- (void) showFindPanel:sender
{
    FIFindController *findController;

    findController = [FIFindController sharedFindController];
    [findController setWindowController:windowController];
    [findController showWindow:self];
}

//======================================================================
// Services
//======================================================================

- (void) submitToFiendDock:(NSPasteboard *)pasteboard userData:(NSData *)data error:(NSString **)error
{
    NSArray *filenames;

    if ([pasteboard availableTypeFromArray:[NSArray arrayWithObject:NSFilenamesPboardType]] != nil)
    {
        filenames = [pasteboard propertyListForType:NSFilenamesPboardType];
        //[self addApplicationsToCurrentLevel:filenames];
    }
}

//======================================================================
//" Dock paths
//======================================================================

- (NSString *) dockFilename
{
    return [NSString stringWithFormat:@"%@.FiendDock", [[NSScreen mainScreen] screenSizeString]];
}

//----------------------------------------------------------------------

- (NSString *) dockPath
{
    return [NSString stringWithFormat:@"%@/Library/Fiend/%@", NSHomeDirectory (), [self dockFilename]];
}

//======================================================================
//" NSApplication delegate
//======================================================================

- (BOOL) applicationShouldOpenUntitledFile:(NSApplication *)sender
{
    return NO;
}

//======================================================================
//" NSWorkspace workaround (for RDR1, RDR2)
//======================================================================

- (void) forwardWorkspaceNotification:(NSNotification *)aNotification
{
    //NSLog (@"notification: %@", aNotification);
    [[NSNotificationCenter fixedWorkspaceCenter] postNotification:aNotification];
}

//======================================================================
// Menu Actions
//======================================================================

- (void) addWorkspaceIcon:sender
{
    [windowController addWorkspaceIcon:sender];
}

//----------------------------------------------------------------------

- (void) rotateLevel:sender
{
    [windowController rotateLevel:sender];
}

//----------------------------------------------------------------------

- (void) addLevel:sender
{
    [windowController addLevel:sender];
}

//----------------------------------------------------------------------

- (void) clearLevel:sender
{
    [windowController clearLevel:sender];
}

//----------------------------------------------------------------------

- (void) deleteLevel:sender
{
    [windowController deleteLevel:sender];
}

//----------------------------------------------------------------------

- (void) nextLevel:sender
{
    [windowController nextLevel:sender];
}

//----------------------------------------------------------------------

- (void) previousLevel:sender
{
    [windowController previousLevel:sender];
}

//----------------------------------------------------------------------

- (void) toggleDockVisible:sender
{
    [windowController toggleDockVisible:sender];
}

//----------------------------------------------------------------------

- (void) toggleDockSunken:sender
{
    [windowController toggleDockSunken:sender];
}

//----------------------------------------------------------------------

- (void) toggleDockLocked:sender
{
    [windowController toggleDockLocked:sender];
}

//----------------------------------------------------------------------

- (void) updateIcons:sender
{
    [windowController updateIcons:sender];
}

//----------------------------------------------------------------------

- (void) cut:sender
{
    [windowController cut:sender];
}

//----------------------------------------------------------------------

- (void) copy:sender
{
    [windowController copy:sender];
}

//----------------------------------------------------------------------

- (void) paste:sender
{
    [windowController paste:sender];
}

//----------------------------------------------------------------------

- (void) clear:sender
{
    [windowController clear:sender];
}

//----------------------------------------------------------------------

- (void) selectAll:sender
{
    [windowController selectAll:sender];
}

//======================================================================
// Other
//======================================================================

- (NSUndoManager *) undoManager
{
    DSTART;
    DEND;
    return nil;
}

//----------------------------------------------------------------------

// XXX: Maybe we should try firstResponder first, and then the dockWindow...

- (void) undo:sender
{
    [[windowController dockWindow] undo:sender];
}

//----------------------------------------------------------------------

- (void) redo:sender
{
    [[windowController dockWindow] redo:sender];
}

//----------------------------------------------------------------------

- (BOOL) validateMenuItem:(NSMenuItem *)menuItem
{
    SEL action = [menuItem action];
    BOOL valid;

    if ([windowController respondsToSelector:action] == YES)
    {
        //NSLog (@"action: %@", NSStringFromSelector (action));
        valid = [windowController validateMenuItem:menuItem];
    }
    else
    {
        valid = [[windowController dockWindow] validateMenuItem:menuItem];
    }

    return valid;
}

//======================================================================
// Auto save timer
//======================================================================

// We'll just save the dock every 5 minutes (if it has changed.)
// XXX: If this is configurable in preferences window, we need to know
// about changes.

- (void) startAutoSaveTimer
{
    double autoSaveInterval;
    
    [self stopAutoSaveTimer];

    autoSaveInterval = [[OFUserDefaults sharedUserDefaults] floatForKey:DK_AutoSaveInterval];
    autoSaveTimer = [[NSTimer scheduledTimerWithTimeInterval:autoSaveInterval
                              target:self
                              selector:@selector (fireAutoSaveTimer:)
                              userInfo:nil
                              repeats:YES] retain];
}

//----------------------------------------------------------------------

- (void) stopAutoSaveTimer
{
    if (autoSaveTimer != nil)
    {
        [autoSaveTimer invalidate];
        SNRelease (autoSaveTimer);
    }
}

//----------------------------------------------------------------------

- (void) fireAutoSaveTimer:(NSTimer *)timer
{
    if ([document isDocumentEdited] == YES)
        [document saveDocument:self];
}

//----------------------------------------------------------------------

- (void) terminate:sender
{
    // NSApp calls the NSDocumentController's _shouldTerminate (which shows a dialog
    // box if there are unsaved documents) before it calls applicationShouldTerminate:,
    // so we otherwise don't have a chance of saving the dock to avoid the dialog.

    [self stopAutoSaveTimer];
    if ([document isDocumentEdited] == YES)
        [document saveDocument:self];

    [NSApp terminate:sender];
}

//----------------------------------------------------------------------

- (void) findNext:sender
{
    [windowController findNext:sender];
}

//----------------------------------------------------------------------

- (void) findPrevious:sender
{
    [windowController findPrevious:sender];
}

//----------------------------------------------------------------------

- (void) _moveDocksToNewFilenames
{
    NSFileManager *fileManager;
    NSEnumerator *filenameEnumerator;
    NSString *libraryPath, *filename, *basename, *newname;
    NSRange prefixRange;

    fileManager = [NSFileManager defaultManager];
    libraryPath = [NSString stringWithFormat:@"%@/Library/Fiend", NSHomeDirectory ()];
    filenameEnumerator = [[fileManager directoryContentsAtPath:libraryPath] objectEnumerator];
    while ((filename = [filenameEnumerator nextObject]))
    {
        prefixRange = [filename rangeOfString:@"FiendDock."];
        if (prefixRange.length > 0 && prefixRange.location == 0)
        {
            NSString *oldPath, *newPath;
            basename = [filename substringFromIndex:prefixRange.length];
            newname = [NSString stringWithFormat:@"%@.FiendDock", basename];
            newPath = [NSString stringWithFormat:@"%@/%@", libraryPath, newname];
            if ([fileManager fileExistsAtPath:newPath] == NO)
            {
                BOOL okay;
                oldPath = [NSString stringWithFormat:@"%@/%@", libraryPath, filename];
                okay = [fileManager movePath:oldPath toPath:newPath handler:nil];
                if (okay == YES)
                    NSLog (@"Moved old dock (%@) to new dock (%@).", filename, newname);
                else
                    NSLog (@"Could not move old dock (%@) to new dock (%@).", filename, newname);
            }
        }
    }
}

@end
