#import "Fiend.h"

RCS_ID ("$Id$");

#import "FIDockView.h"

@implementation FIDockView

static NSImage *_fiendImage = nil;
static NSImage *_alternateFiendImage = nil;

static NSImage *_leftTriangleImage = nil;
static NSImage *_leftTriangleHImage = nil;
static NSImage *_diamondImage = nil;
static NSImage *_diamondHImage = nil;
static NSImage *_rightTriangleImage = nil;
static NSImage *_rightTriangleHImage = nil;


static NSSize _fiendImageSize;
static NSSize _alternateFiendImageSize;
         
static NSSize _leftTriangleImageSize;
static NSSize _leftTriangleHImageSize;
static NSSize _diamondImageSize;
static NSSize _diamondHImageSize;
static NSSize _rightTriangleImageSize;
static NSSize _rightTriangleHImageSize;

+ (void) initialize
{
    static BOOL initialized = NO;

    [super initialize];
    if (initialized == YES)
        return;
    initialized = YES;
    
    // XXX: What about scaling for the rest of the images?
    _fiendImage = [[NSImage imageNamed:@"FiendIcon" inBundleForClass:self] retain];
    _fiendImageSize = [_fiendImage size];
    [_fiendImage setScalesWhenResized:YES]; 
   
    _alternateFiendImage = [[NSImage imageNamed:@"AltFiendIcon" inBundleForClass:self] retain];
    _alternateFiendImageSize = [_alternateFiendImage size];
    [_alternateFiendImage setScalesWhenResized:YES];

    _leftTriangleImage = [[NSImage imageNamed:@"leftTriangle" inBundleForClass:self] retain];
    _leftTriangleImageSize = [_leftTriangleImage size];
    [_leftTriangleImage setScalesWhenResized:YES];

    _leftTriangleHImage = [[NSImage imageNamed:@"leftTriangleH" inBundleForClass:self] retain];
    _leftTriangleHImageSize = [_leftTriangleHImage size];
    [_leftTriangleHImage setScalesWhenResized:YES];

    _diamondImage = [[NSImage imageNamed:@"diamond" inBundleForClass:self] retain];
    _diamondImageSize = [_diamondImage size];
    [_diamondImage setScalesWhenResized:YES];

    _diamondHImage = [[NSImage imageNamed:@"diamondH" inBundleForClass:self] retain];
    _diamondHImageSize = [_diamondHImage size];
    [_diamondHImage setScalesWhenResized:YES];

    _rightTriangleImage = [[NSImage imageNamed:@"rightTriangle" inBundleForClass:self] retain];
    _rightTriangleImageSize = [_rightTriangleImage size];
    [_rightTriangleImage setScalesWhenResized:YES];

    _rightTriangleHImage = [[NSImage imageNamed:@"rightTriangleH" inBundleForClass:self] retain];
    _rightTriangleHImageSize = [_rightTriangleHImage size];
    [_rightTriangleHImage setScalesWhenResized:YES];

    [self adjustImageSizes];
}

//----------------------------------------------------------------------

#define FIScaleSize(aSize,scaleFactor) NSMakeSize (aSize.width*scaleFactor,aSize.height*scaleFactor)

+ (void) adjustImageSizes
{
    float tileWidth;
    float scaleFactor;

    tileWidth = [[OFUserDefaults sharedUserDefaults] floatForKey:DK_TileSize];
    scaleFactor = tileWidth / 64.0;

    [_fiendImage setSize:FIScaleSize (_fiendImageSize, scaleFactor)];
    [_alternateFiendImage setSize:FIScaleSize (_alternateFiendImageSize, scaleFactor)];
    [_leftTriangleImage setSize:FIScaleSize (_leftTriangleImageSize, scaleFactor)];
    [_leftTriangleHImage setSize:FIScaleSize (_leftTriangleHImageSize, scaleFactor)];
    [_diamondImage setSize:FIScaleSize (_diamondImageSize, scaleFactor)];
    [_diamondHImage setSize:FIScaleSize (_diamondHImageSize, scaleFactor)];
    [_rightTriangleImage setSize:FIScaleSize (_rightTriangleImageSize, scaleFactor)];
    [_rightTriangleHImage setSize:FIScaleSize (_rightTriangleHImageSize, scaleFactor)];
}

//----------------------------------------------------------------------

- initWithFrame:(NSRect)frameRect
{
    if ([super initWithFrame:frameRect] == nil)
        return nil;

    [self registerForDraggedTypes:[NSArray arrayWithObject:NSFilenamesPboardType]];

    previousLevelButton = [[NSButton allocWithZone:[self zone]] initWithFrame:NSMakeRect (51, 4, 9, 9)];
    [previousLevelButton setImage:_leftTriangleImage];
    [previousLevelButton setAlternateImage:_leftTriangleHImage];
    [previousLevelButton setBordered:NO];
    [previousLevelButton setImagePosition:NSImageOnly];
    [previousLevelButton setButtonType:NSMomentaryChangeButton];
    [previousLevelButton setEnabled:YES];

    [previousLevelButton setTarget:self];
    [previousLevelButton setAction:@selector (buttonAction:)];
    [self addSubview:previousLevelButton];
    //[previousLevelButton release];

    activateButton = [[NSButton allocWithZone:[self zone]] initWithFrame:NSMakeRect (51, 4+9, 9, 10)];
    [activateButton setImage:_diamondImage];
    [activateButton setAlternateImage:_diamondHImage];
    [activateButton setBordered:NO];
    [activateButton setImagePosition:NSImageOnly];
    [activateButton setButtonType:NSMomentaryChangeButton];
    [activateButton setEnabled:YES];

    [activateButton setTarget:self];
    [activateButton setAction:@selector (buttonAction:)];
    [self addSubview:activateButton];
    //[activateButton release];

    nextLevelButton = [[NSButton allocWithZone:[self zone]] initWithFrame:NSMakeRect (51, 4+9+10, 9, 9)];
    [nextLevelButton setImage:_rightTriangleImage];
    [nextLevelButton setAlternateImage:_rightTriangleHImage];
    [nextLevelButton setBordered:NO];
    [nextLevelButton setImagePosition:NSImageOnly];
    [nextLevelButton setButtonType:NSMomentaryChangeButton];

    [nextLevelButton setTarget:self];
    [nextLevelButton setAction:@selector (buttonAction:)];
    [self addSubview:nextLevelButton];
    //[nextLevelButton release];



    dockNameCell = [[NSTextFieldCell allocWithZone:[self zone]] initTextCell:@"1-1"];
    [dockNameCell setAlignment:NSLeftTextAlignment];
    [dockNameCell setDrawsBackground:NO];
    [dockNameCell setTextColor:[NSColor blackColor]];
    [dockNameCell setEditable:NO];
    [dockNameCell setEnabled:YES];
    [dockNameCell setWraps:NO]; // For RDR2
    //[dockNameCell setFont:[NSFont controlContentFontOfSize:10]];

    dockNumberCell = [[NSTextFieldCell allocWithZone:[self zone]] initTextCell:@"1"];
    [dockNumberCell setAlignment:NSLeftTextAlignment];
    [dockNumberCell setDrawsBackground:NO];
    [dockNumberCell setTextColor:[NSColor blackColor]];
    [dockNumberCell setEditable:NO];
    [dockNumberCell setEnabled:YES];
    //[dockNumberCell setFont:[NSFont controlContentFontOfSize:10]];

    // XXX: Adjust cell fonts
    {
        OFUserDefaults *defaults;
        NSString *fontName;
        float fontSize;

        defaults = [OFUserDefaults sharedUserDefaults];
        fontName = [defaults objectForKey:DK_DockLevelNameFont];
        fontSize = ([defaults boolForKey:DK_UseBigDockFont] == YES) ? 12 : 10;
        [dockNameCell setFont:[NSFont fontWithName:fontName size:fontSize]];
        [dockNumberCell setFont:[NSFont fontWithName:fontName size:fontSize - 1]];
        //[dockNameCell setFont:[NSFont controlContentFontOfSize:fontSize]];
        //[dockNumberCell setFont:[NSFont controlContentFontOfSize:fontSize - 1]];
    }

    lockedPath = [[NSBezierPath bezierPath] retain];
    [lockedPath appendBezierPathWithArcWithCenter:NSMakePoint (55, 38) radius:1.8 startAngle:0 endAngle:360];

    nonretainedDatasource = nil;
    nonretainedDelegate = nil;

    [[NSNotificationCenter defaultCenter] addObserver:self
                                          selector:@selector (styleDidChange:)
                                          name:FIPreferencesShowLevelNumberDidChangeNotification
                                          object:nil];

    return self;
}

//----------------------------------------------------------------------

- (void) dealloc
{
    [[NSNotificationCenter defaultCenter] removeObserver:self];
    
    SNRelease (lockedPath);
    SNRelease (dockNumberCell);
    SNRelease (dockNameCell);
    SNRelease (nextLevelButton);
    SNRelease (activateButton);
    SNRelease (previousLevelButton);

    [super dealloc];
}

//----------------------------------------------------------------------

- (void) drawRect:(NSRect)rect
{
    OFUserDefaults *defaults;
    NSRect boundsRect, dockNameFrame, dockNumberFrame;

    [super drawRect:rect];

    defaults = [OFUserDefaults sharedUserDefaults];
    boundsRect = [self bounds];

    
    if (nonretainedDatasource != nil)
    {
        [dockNumberCell setStringValue:[nonretainedDatasource currentLevelNumber]];
        [dockNameCell setStringValue:[nonretainedDatasource currentLevelName]];
    }

    if (_fiendImage != nil)
    {
        NSImage *tileImage;
        NSSize imageSize, dockNameCellSize;
        NSPoint aPoint;

#if 0
        tileImage = ([self isDockVisible] == YES) ? _fiendImage : _alternateFiendImage;
        imageSize = [tileImage size];
#else
        if ([self isDockVisible] == YES)
        {
            tileImage = _fiendImage;
            imageSize = _fiendImageSize;
        }
        else
        {
            tileImage = _alternateFiendImage;
            imageSize = _alternateFiendImageSize;
        }
#endif
        dockNameCellSize = [dockNameCell cellSize];
        aPoint.x = rint ((64 - 6 - imageSize.width) / 2.0);
        aPoint.y = rint ((64 - dockNameCellSize.height - imageSize.height) / 2.0);
        [tileImage compositeToPoint:aPoint operation:NSCompositeSourceOver];
    }

    dockNameFrame.size = [dockNameCell cellSize];
    dockNameFrame.origin.x = 2;
    // Skip over the top pixel border (should be 2 pixels...)
    dockNameFrame.origin.y = boundsRect.size.height - 1 - dockNameFrame.size.height;
    [dockNameCell drawWithFrame:dockNameFrame inView:self];

    if ([defaults boolForKey:DK_ShowLevelNumber] == YES)
    {
        // Skip over the 2 pixel tile border
        dockNumberFrame.origin = NSMakePoint (2, 2); 
        dockNumberFrame.size = [dockNumberCell cellSize];
        [dockNumberCell drawWithFrame:dockNumberFrame inView:self];
    }

    if ([self isDockLocked] == YES)
    {
        [[NSColor redColor] set];
        [lockedPath fill];
        [[NSColor blackColor] set];
        PSsetlinewidth (0);
        [lockedPath stroke];
    }
}

//----------------------------------------------------------------------

- (void) mouseDown:(NSEvent *)theEvent
{
    unsigned int modifierFlags;
    int clickCount;

    modifierFlags = [theEvent modifierFlags];
    clickCount = [theEvent clickCount];

    if (clickCount == 1)
    {
        [nonretainedDelegate dockView:self startedDragging:theEvent];
    }
}

//----------------------------------------------------------------------

- (void) mouseUp:(NSEvent *)theEvent
{
    unsigned int modifierFlags;
    int control, command, alternate, shift;
    int clickCount;

    modifierFlags = [theEvent modifierFlags];
    clickCount = [theEvent clickCount];
    control = modifierFlags & NSControlKeyMask;
    command = modifierFlags & NSCommandKeyMask;
    alternate = modifierFlags & NSAlternateKeyMask;
    shift = modifierFlags & NSShiftKeyMask;
    
    //NSLog (@"modifierFlags: %08x", modifierFlags);
    if (modifierFlags & NSAlternateKeyMask)
    {
        [nonretainedDelegate toggleDockSunken:self];
    }
    else if (modifierFlags & NSControlKeyMask)
    {
        [nonretainedDelegate toggleDockLocked:self];
    }
    else if (clickCount == 2 && command)
    {
        [nonretainedDelegate activateApplication:self]; // XXX: And hide others...
    }
    else if (!control & !command & !alternate & !shift)
    {
        if (clickCount == 1
            && [NSApp nextEventMatchingMask:NSLeftMouseDownMask
                      untilDate:[NSDate dateWithTimeIntervalSinceNow:0.2]
                      inMode:NSModalPanelRunLoopMode
                      dequeue:NO] == nil)
        {
            [nonretainedDelegate toggleDockVisible:self];
        }
        else if (clickCount == 2)
        {
            // Most general case last.
            [nonretainedDelegate activateWorkspace:self];
        }
    }
}

//======================================================================
// Attributes
//======================================================================

- datasource
{
    return nonretainedDatasource;
}

//----------------------------------------------------------------------

- (void) setDatasource:newDatasource
{
    if (newDatasource == nonretainedDatasource)
        return;

    nonretainedDatasource = newDatasource;
    [self setNeedsDisplay:YES];
}

//----------------------------------------------------------------------

- delegate
{
    return nonretainedDelegate;
}

//----------------------------------------------------------------------

- (void) setDelegate:newDelegate
{
    if (newDelegate == nonretainedDelegate)
        return;

    nonretainedDelegate = newDelegate;
}

//======================================================================
// Covers for datasource methods.
//======================================================================

- (BOOL) isDockVisible
{
    return (nonretainedDatasource == nil) ? YES : [nonretainedDatasource isDockVisible];
}

//----------------------------------------------------------------------

- (BOOL) isDockLocked
{
    return (nonretainedDatasource == nil) ? NO : [nonretainedDatasource isDockLocked];
}

//======================================================================
//" Button actions
//======================================================================

- (void) buttonAction:sender
{
    if (sender == previousLevelButton)
    {
        [nonretainedDelegate previousLevel:sender];
    }
    else if (sender == nextLevelButton)
    {
        [nonretainedDelegate nextLevel:sender];
    }
    else if (sender == activateButton)
    {
        [nonretainedDelegate activateApplication:sender];
    }
}

//======================================================================
// NSDraggingDestination
//======================================================================

- (BOOL) prepareForDragOperation:(id <NSDraggingInfo>)sender
{
    // App in dock, but not being dragged from out dock: reject
    // App in dock, but being dragged from out dock: remove it from dock

    // Reject a multiple selection if any of the applications are already on the dock
    return YES;
}

//----------------------------------------------------------------------

- (BOOL) performDragOperation:(id <NSDraggingInfo>)sender
{
    return YES;
}

//----------------------------------------------------------------------

- (void) concludeDragOperation:(id <NSDraggingInfo>)sender
{
    NSPasteboard *draggingPasteboard;
    NSArray *filenames;
    
    draggingPasteboard = [sender draggingPasteboard];
    //NSLog (@"draggingPasteboard: %@", draggingPasteboard);

    //NSLog (@"available types: %@", [draggingPasteboard types]);

    if ([draggingPasteboard availableTypeFromArray:[NSArray arrayWithObject:NSFilenamesPboardType]] != nil)
    {
        filenames = [draggingPasteboard propertyListForType:NSFilenamesPboardType];
        //NSLog (@"[%@] filenames: %@", NSStringFromClass ([filenames class]), filenames);

        [nonretainedDelegate dockView:self acceptedDraggedApplications:filenames];
    }
}

//----------------------------------------------------------------------

- (unsigned int) draggingEntered:(id <NSDraggingInfo>)sender
{
    return NSDragOperationCopy;
}

//----------------------------------------------------------------------

- (unsigned int) draggingUpdated:(id <NSDraggingInfo>)sender
{
    return NSDragOperationCopy;
}

//----------------------------------------------------------------------

- (void) draggingExited:(id <NSDraggingInfo>)sender
{
}

//======================================================================
//" Handling preference notifications
//======================================================================

- (void) styleDidChange:(NSNotification *)aNotification
{
    [self setNeedsDisplay:YES];
}

@end
