#import "Fiend.h"

RCS_ID ("$Id$");

#import "FIFindController.h"

#import "FIWindowController.h"

@implementation FIFindController

+ sharedFindController
{
    static FIFindController *instance = nil;

    if (instance == nil)
    {
        instance = [[FIFindController alloc] init];
        [instance setWindowFrameAutosaveName:@"FindPanel"];
    }

    return instance;
}

//----------------------------------------------------------------------

- init
{
    if ([super initWithWindowNibName:@"FindPanel"] == nil)
        return nil;

    nonretainedWindowController = nil;

    return self;
}

//----------------------------------------------------------------------

- (void) dealloc
{
    [super dealloc];
}

//----------------------------------------------------------------------

- (void) showWindow:sender
{
    [self window]; // Make sure the window is loaded.
    [self restoreFindText];

    [super showWindow:sender];
}

//----------------------------------------------------------------------

- (void) windowDidLoad
{
    OFUserDefaults *defaults;
    
    [super windowDidLoad];

    defaults = [OFUserDefaults sharedUserDefaults];
    [ignoreCaseSwitch setState:[defaults boolForKey:DK_FindIgnoresCase]];
    [targetMatrix selectCellWithTag:([defaults boolForKey:DK_FindApplications] == YES) ? 0 : 1];
}

//----------------------------------------------------------------------

- (FIWindowController *) windowController
{
    return nonretainedWindowController;
}

//----------------------------------------------------------------------

- (void) setWindowController:(FIWindowController *)newWindowController
{
    nonretainedWindowController = newWindowController;
}

//----------------------------------------------------------------------

- (void) findTargetAction:sender
{
    OFUserDefaults *defaults;
    int tag;

    defaults = [OFUserDefaults sharedUserDefaults];
    tag = [[targetMatrix selectedCell] tag];
    [defaults setBool:((tag == 0) ? YES : NO) forKey:DK_FindApplications];
}

//----------------------------------------------------------------------

- (void) toggleIgnoreCase:sender
{
    OFUserDefaults *defaults;

    defaults = [OFUserDefaults sharedUserDefaults];
    [defaults setBool:[ignoreCaseSwitch state] forKey:DK_FindIgnoresCase];
}

//----------------------------------------------------------------------

- (void) textAction:sender
{
    [self saveFindText];
}

//----------------------------------------------------------------------

- (void) findNext:sender
{
    BOOL isFirstResponder;
    id firstResponder = nil;
    
    isFirstResponder = [findTextField currentEditor] != nil;
    if (isFirstResponder == YES)
        firstResponder = findTextField;
    
    [[self window] endEditingFor:nil];
    if (firstResponder != nil)
        [[self window] makeFirstResponder:firstResponder];

    [nonretainedWindowController findNext:self];
}

//----------------------------------------------------------------------

- (void) findPrevious:sender
{
    BOOL isFirstResponder;
    id firstResponder = nil;
    
    isFirstResponder = [findTextField currentEditor] != nil;
    if (isFirstResponder == YES)
        firstResponder = findTextField;
    
    [[self window] endEditingFor:nil];
    if (firstResponder != nil)
        [[self window] makeFirstResponder:firstResponder];

    [nonretainedWindowController findPrevious:self];
}

//----------------------------------------------------------------------

- (void) saveFindText
{
    NSPasteboard *findPasteboard;
    NSString *findString;

    findString = [findTextField stringValue];
    OBPOSTCONDITION (findString != nil);
    findPasteboard = [NSPasteboard pasteboardWithName:NSFindPboard];
    [findPasteboard declareTypes:[NSArray arrayWithObject:NSStringPboardType] owner:nil];
    [findPasteboard setString:findString forType:NSStringPboardType];
}

//----------------------------------------------------------------------

- (void) restoreFindText
{
    NSPasteboard *findPasteboard;
    NSString *availableType;
    NSString *findString = nil;

    findPasteboard = [NSPasteboard pasteboardWithName:NSFindPboard];
    availableType = [findPasteboard availableTypeFromArray:[NSArray arrayWithObject:NSStringPboardType]];
    if (availableType == NSStringPboardType)
    {
        findString = [findPasteboard stringForType:NSStringPboardType];
        SNLog (findString);
    }
    if (findString == nil)
        findString = @"";
    [findTextField setStringValue:findString];
}

@end
