#import "Fiend.h"

RCS_ID ("$Id$");

#import "FILauncher.h"

DEFINE_NSSTRING (FIFailedToLaunchApplicationNotificaiton);
DEFINE_NSSTRING (FILauncherApplicationPath);

#define FILNoData (0)
#define FILHasData (1)

@implementation FILauncher

+ sharedLauncher
{
    static FILauncher *instance = nil;

    if (instance == nil)
    {
        instance = [[FILauncher alloc] init];
    }

    return instance;
}

//----------------------------------------------------------------------

- init
{
    if ([super init] == nil)
        return nil;

    launchQueue = [[NSMutableArray allocWithZone:[self zone]] initWithCapacity:4];
    queueLock = [[NSConditionLock allocWithZone:[self zone]] initWithCondition:FILNoData];
    workspaceLock = [[NSLock allocWithZone:[self zone]] init];

    [NSThread detachNewThreadSelector:@selector (launcherThread:) toTarget:self withObject:nil];

    return self;
}

//----------------------------------------------------------------------

- (void) dealloc
{
    SNRelease (launchQueue);

    [super dealloc];
}

//----------------------------------------------------------------------

- (void) launchApplication:(NSString *)path autolaunch:(BOOL)autolaunchFlag
{
    NSMutableDictionary *launchDictionary;

    launchDictionary = [[NSMutableDictionary allocWithZone:[self zone]] initWithCapacity:4];
    [launchDictionary setObject:path forKey:@"FILPath"];
    [launchDictionary setObject:[NSNumber numberWithBool:autolaunchFlag] forKey:@"FILAutolaunch"];
    //SNLog (launchDictionary);
    [queueLock lock];
    [launchQueue addObject:launchDictionary];
    //SNLog (launchQueue);
    [queueLock unlockWithCondition:FILHasData];
    [launchDictionary release];
}

//----------------------------------------------------------------------

- (void) openFiles:(NSArray *)files withApplication:(NSString *)path
{
    NSMutableDictionary *launchDictionary;

    launchDictionary = [[NSMutableDictionary allocWithZone:[self zone]] initWithCapacity:4];
    [launchDictionary setObject:path forKey:@"FILPath"];
    [launchDictionary setObject:[NSNumber numberWithBool:NO] forKey:@"FILAutolaunch"];
    [launchDictionary setObject:files forKey:@"FILFilesToOpen"];
    //SNLog (launchDictionary);
    [queueLock lock];
    [launchQueue addObject:launchDictionary];
    //SNLog (launchQueue);
    [queueLock unlockWithCondition:FILHasData];
    [launchDictionary release];
}

//----------------------------------------------------------------------

- (void) _openFiles:(NSArray *)files withApplication:(NSString *)path
{
    NSEnumerator *filenameEnumerator;
    NSString *filename;
    NSWorkspace *workspace;
    BOOL okay;
    
    //SNLog (files);
    //SNLog (path);

    workspace = [NSWorkspace sharedWorkspace];
    filenameEnumerator = [files objectEnumerator];
    while (filename = [filenameEnumerator nextObject])
    {
        //SNLog (filename);
        [workspaceLock lock];
        // Deactivating is not thread-safe in DR2...
        okay = [workspace openFile:filename withApplication:path andDeactivate:NO];
        [workspaceLock unlock];
        if (okay == NO)
            NSLog (@"Could not open file %@ with %@", filename, path);
        //SNLogBool (okay);
    }
}

//----------------------------------------------------------------------

- (void) launcherThread:sender
{
    NSAutoreleasePool *pool;
    BOOL isEmpty, didLaunch;
    NSDictionary *launchDictionary;
    NSWorkspace *workspace;
    NSArray *filesToOpen;

    pool = [[NSAutoreleasePool alloc] init];

    workspace = [NSWorkspace sharedWorkspace];

    while (1)
    {
        [queueLock lockWhenCondition:FILHasData];
        launchDictionary = [[launchQueue objectAtIndex:0] retain];
        [launchQueue removeObjectAtIndex:0];
        //SNLog (launchQueue);
        isEmpty = ([launchQueue count] == 0) ? YES : NO;
        [queueLock unlockWithCondition:(isEmpty == YES) ? FILNoData : FILHasData];

        //SNLog (launchDictionary);
        filesToOpen = [launchDictionary objectForKey:@"FILFilesToOpen"];
        if (filesToOpen == nil)
        {
            //NSLog (@"launching %@", [launchDictionary objectForKey:@"FILPath"]);
            [workspaceLock lock];
            didLaunch = [workspace launchApplication:[launchDictionary objectForKey:@"FILPath"]
                                   showIcon:YES
                                   autolaunch:[[launchDictionary objectForKey:@"FILAutolaunch"] boolValue]];
            [workspaceLock unlock];
            //NSLog (@"launched %@", [launchDictionary objectForKey:@"FILPath"]);
            if (didLaunch == NO)
            {
                NSLog (@"Didn't launch %@", [launchDictionary objectForKey:@"FILPath"]);
            }
        }
        else
        {
            [self _openFiles:filesToOpen withApplication:[launchDictionary objectForKey:@"FILPath"]];
            //NSLog (@"opened with %@", [launchDictionary objectForKey:@"FILPath"]);
        }

        [launchDictionary release];
    }

    [pool release];
}

//----------------------------------------------------------------------

- (NSImage *) iconForFile:(NSString *)fullPath
{
    NSImage *icon;
    
    [workspaceLock lock];
    icon = [[NSWorkspace sharedWorkspace] iconForFile:fullPath];
    [workspaceLock unlock];

    return icon;
}

@end
