//
// $Id$
//

#import <AppKit/AppKit.h>

@class FIApplication;
@class FILevel;
@class FITile;
@class FIWindowController;

@interface FILevelRepresentation : OFObject
{
    FIWindowController *nonretainedWindowController;
    FILevel *level;
    NSMutableArray *tiles;
    BOOL isVisible;
}

- initWithLevel:(FILevel *)aLevel;
- (void) dealloc;

//======================================================================
//" Controlling general attributes
//======================================================================

- (FILevel *) level;

- (BOOL) isVisible;
- (void) setIsVisible:(BOOL)flag;

- (FIWindowController *) windowController;
- (void) setWindowController:(FIWindowController *)newWindowController;

//======================================================================
//" Debugging
//======================================================================

- (NSMutableDictionary *) debugDictionary;

//======================================================================
//" Handling Workspace notifications
//======================================================================

- (void) applicationWillLaunch:(NSNotification *)aNotification;
- (void) applicationDidLaunch:(NSNotification *)aNotification;
- (void) applicationDidTerminate:(NSNotification *)aNotification;

//======================================================================
//" Controlling tiles
//======================================================================

- (NSArray *) tiles;
- (FITile *) tileForApplication:(FIApplication *)application;
- (FITile *) tileAtDockLocation:(DockLocation)location;

- (void) addTile:(FITile *)tile checkActive:(BOOL)checkActive;
- (void) _removeTile:(FITile *)tile;
- (void) removeTile:(FITile *)tile;

//======================================================================
//" Handling model notifications
//======================================================================

- (void) levelDidAddApplication:(NSNotification *)aNotification;
- (void) levelDidRemoveApplication:(NSNotification *)aNotification;

//======================================================================
//" Controlling tile windows
//======================================================================

- (void) setTileWindowLevel:(int)windowLevel;

// XXX: make these two private.  Should use setIsVisible: instead.
- (void) hideLevel;
- (void) unhideLevel;
- (void) repositionRelativeToPoint:(NSPoint)origin;

//======================================================================
//" Other
//======================================================================

- (void) checkForActiveApplications:(NSDictionary *)activeApplications;
- (void) performAutoLaunch;

- (void) updateFileIcons;
- (void) updateFileIconsIfNeeded;

@end
