//
// $Id$
//

#import <AppKit/AppKit.h>
#import <OmniAppKit/OmniAppKit.h>
#import "FITileViewDatasourceProtocol.h"
#import "FITileViewDelegateProtocol.h"
#import "FITileDelegateProtocol.h"

@class FIApplication;
@class FIAppInfo;
@class FITileView;
@class FITileWindow;
@class FIDock;

extern NSString *FITileIconDidChangeNotification;

typedef enum _ApplicationState
{
    ApplicationInactiveState,
    ApplicationLaunching1State,
    ApplicationLaunching2State,
    ApplicationActiveState
} ApplicationState;

@interface FITile : OFObject <FITileViewDatasource, FITileViewDelegate>
{
    FIApplication *application;

    FITileWindow *tileWindow;
    FITileView *tileView; // XXX: should be FITileView

    BOOL hasLoadedFileIcon;
    NSImage *fileIcon;
    
    BOOL isVisible;
    BOOL isSelected;
    BOOL tileLaunchedApplication;  //" If YES, then activate application when it finished launching.

    ApplicationState state;
    NSTimer *appInfoTimer;
    FIAppInfo *appInfo;
    BOOL hasCheckedIfActive;

    NSTimer *animationTimer;
    NSImage *animationImage;

    NSTimer *autoLaunchTimer;

    id <NSObject, FITileDelegate> nonretainedDelegate;
}

- initWithApplication:(FIApplication *)anApplication;
- (void) dealloc;
- (void) stopTimers;

//======================================================================
//" Controlling the delegate
//======================================================================

- delegate;
- (void) setDelegate:newDelegate;

//======================================================================
//" Controlling general attributes
//======================================================================

- (FIApplication *) application;
- (FIDock *) dock;

- (BOOL) isSelected;
- (void) setIsSelected:(BOOL)flag;

- (BOOL) hasCheckedIfActive;

//======================================================================
//" Handling state changes
//======================================================================

- (void) enterInactiveState;
- (void) enterLaunching1State;
- (void) enterLaunching2State;
- (void) enterActiveState:(FIAppInfo *)anAppInfo;

//======================================================================
//" Managing app info timer
//======================================================================

- (void) startAppInfoTimer;
- (void) stopAppInfoTimer;
- (void) fireAppInfoTimer:(NSTimer *)aTimer;

//======================================================================
//" Managing animation timer
//======================================================================

- (void) startAnimationTimer;
- (void) stopAnimationTimer;
- (void) fireAnimationTimer:(NSTimer *)aTimer;

//======================================================================
//" Managing delayed auto launch timer
//======================================================================

- (void) startAutoLaunchTimer;
- (void) stopAutoLaunchTimer;
- (void) fireAutoLaunchTimer:(NSTimer *)aTimer;

//======================================================================
//" Controlling the source tile
//======================================================================

- (void) hideSourceTile;
- (void) unhideSourceTile;

//======================================================================
//" FITileView datasource
//======================================================================

- (NSImage *) animationImage;
- (BOOL) shouldAutoLaunch;
- (BOOL) shouldAnimate;
- (BOOL) isPersistent;
- (BOOL) isRunning;
- (BOOL) isGhosted;
//- (BOOL) isSelected;

//======================================================================
//" FITileView delegate
//======================================================================

- (void) tileView:(FITileView *)aTileView singleClicked:(NSEvent *)theEvent;
- (void) tileView:(FITileView *)aTileView doubleClicked:(NSEvent *)theEvent;
- (void) tileView:(FITileView *)aTileView acceptedFiles:(NSArray *)filenames;

- (NSEvent *) trackMouseInRect:(NSRect)rect;

//======================================================================
//" Debugging
//======================================================================

- (NSMutableDictionary *) debugDictionary;

//======================================================================
//" Handling Workspace notifications
//======================================================================

- (void) applicationWillLaunch:(NSNotification *)aNotification;
- (void) applicationDidLaunch:(NSNotification *)aNotification;
- (void) applicationDidTerminate:(NSNotification *)aNotification;

//======================================================================
//" Controlling the file icon
//======================================================================

- (BOOL) hasLoadedFileIcon;
- (NSImage *) fileIcon;
- (void) updateFileIcon;
- (void) updateFileIconIfNeeded;

//======================================================================
//" Controlling the tile window
//======================================================================

- (FITileWindow *) tileWindow;

- (BOOL) isVisible;
- (void) setIsVisible:(BOOL)flag;

// We can use makeObjectsPerformSelector: with these.
- (void) hide;
- (void) unhide;

- (void) repositionRelativeToPoint:(NSPoint)origin;

//======================================================================
//" NSWindow delegate
//======================================================================

- (NSUndoManager *) windowWillReturnUndoManager:(NSWindow *)window;

//======================================================================
//" Handling model notifications
//======================================================================

- (void) applicationDidChange:(NSNotification *)aNotification;
- (void) applicationDidChangeDockLocation:(NSNotification *)aNotification;

//======================================================================
//" Other
//======================================================================

- (void) checkForActiveApplications:(NSDictionary *)activeApplications;
- (void) launchApplicationAndHideOthers:(BOOL)hideOthers;
- (void) performAutoLaunch;

@end
