#import "Fiend.h"

RCS_ID ("$Id$");

#import "FITileView.h"

#import "FiendWraps.h" // For PSWMarkStatus()

@implementation FITileView

static NSImage *_dotsImage = nil;

static NSBezierPath *_autoLaunchPath = nil;
static NSBezierPath *_animatePath = nil;
static NSBezierPath *_persistentPath = nil;
static NSBezierPath *_autoLaunchAndAnimatePath = nil;

+ (void) initialize
{
    static BOOL initialized = NO;

    [super initialize];
    if (initialized == YES)
        return;
    initialized = YES;

    _dotsImage = [[NSImage imageNamed:@"dots" inBundleForClass:self] retain];
    OBPOSTCONDITION (_dotsImage != nil);

    // 64*0.2, 64*0.9, 64*0.2, 64*0.05
    _autoLaunchPath = [[NSBezierPath bezierPathWithRect:NSMakeRect (13, 58, 13, 3)] retain];
    // 64*0.6, 64*0.9, 64*0.2, 64*0.05
    [_autoLaunchPath appendBezierPathWithRect:NSMakeRect (38, 58, 13, 3)];

    // 64*0.4, 64*0.9, 64*0.2, 64*0.05
    _animatePath = [[NSBezierPath bezierPathWithRect:NSMakeRect (25, 58, 13, 3)] retain];

    // 64*0.2, 64*0.9, 64*0.6, 64*0.05
    _autoLaunchAndAnimatePath = [[NSBezierPath bezierPathWithRect:NSMakeRect (13, 58, 38, 3)] retain];

    // [round(64*0.85),1] -> [64-1,round(64*0.15)] -> [64-1,1]
    _persistentPath = [[NSBezierPath bezierPath] retain];
    [_persistentPath moveToPoint:NSMakePoint (55, 2)];
    [_persistentPath lineToPoint:NSMakePoint (62, 9)];
    [_persistentPath lineToPoint:NSMakePoint (62, 1)];
    [_persistentPath closePath];
}

//----------------------------------------------------------------------

- initWithFrame:(NSRect)frameRect
{
    NSNotificationCenter *defaultCenter;
    
    if ([super initWithFrame:frameRect] == nil)
        return nil;

    [self registerForDraggedTypes:[NSArray arrayWithObject:NSFilenamesPboardType]];

    image = nil;
    compositePoint = NSZeroPoint;

    nonretainedDatasource = nil;
    nonretainedDelegate = nil;

    defaultCenter = [NSNotificationCenter defaultCenter];
    [defaultCenter addObserver:self
                   selector:@selector (styleDidChange:)
                   name:FIPreferencesShowFlagCuesDidChangeNotification
                   object:nil];

    [defaultCenter addObserver:self
                   selector:@selector (styleDidChange:)
                   name:FIPreferencesSelectedTileColorDidChangeNotification
                   object:nil];

    [defaultCenter addObserver:self
                   selector:@selector (styleDidChange:)
                   name:FIPreferencesFlagCueColorDidChangeNotification
                   object:nil];

    return self;
}

//----------------------------------------------------------------------

- (void) dealloc
{
    [[NSNotificationCenter defaultCenter] removeObserver:self];

    SNRelease (image);
    
    [super dealloc];
}

//----------------------------------------------------------------------

- (void) drawRect:(NSRect)rect
{
    OFUserDefaults *defaults;
    NSImage *animationImage;
    
    [super drawRect:rect];

    defaults = [OFUserDefaults sharedUserDefaults];

    animationImage = [self animationImage];
    if (animationImage == nil)
    {
        if (image != nil)
        {
            [image compositeToPoint:compositePoint operation:NSCompositeSourceOver];
        }

        if (_dotsImage != nil && [self isRunning] == NO)
        {
            [_dotsImage compositeToPoint:NSZeroPoint operation:NSCompositeSourceOver];
        }

        if ([self isGhosted] == YES)
        {
            [[NSColor colorWithCalibratedWhite:1.0 alpha:0.5] set];
            PScompositerect (2, 2, 60, 60, NSCompositeSourceOver);
        }
    }
    else
    {
        [animationImage compositeToPoint:NSMakePoint (2, 2) operation:NSCompositeCopy];
    }

    if ([defaults boolForKey:DK_ShowFlagCues] == YES)
    {
#if 1
        BOOL shouldAutoLaunch, shouldAnimate, isPersistent;

        shouldAutoLaunch = [self shouldAutoLaunch];
        shouldAnimate = [self shouldAnimate];
        isPersistent = [self isPersistent];

        //[[NSColor greenColor] set];
        [[defaults colorForKey:DK_FlagCueColor] set];
        if (shouldAutoLaunch == YES && shouldAnimate == YES)
            [_autoLaunchAndAnimatePath fill];
        else if (shouldAutoLaunch == YES)
            [_autoLaunchPath fill];
        else if (shouldAnimate == YES)
            [_animatePath fill];

        if (isPersistent == YES)
            [_persistentPath fill];
            

        //[[NSColor blackColor] set];
        [[defaults colorForKey:DK_FlagCueBorderColor] set];
        if (shouldAutoLaunch == YES && shouldAnimate == YES)
            [_autoLaunchAndAnimatePath stroke];
        else if (shouldAutoLaunch == YES)
            [_autoLaunchPath stroke];
        else if (shouldAnimate == YES)
            [_animatePath stroke];

        if (isPersistent == YES)
            [_persistentPath stroke];
#else
        PSWMarkStatus (0, 64, 64, YES, [self shouldAutoLaunch], [self shouldAnimate], [self isPersistent]);
#endif
    }

    if ([self isSelected] == YES)
    {
        PSsetlinewidth (2);
        [[defaults colorForKey:DK_SelectedTileColor] set];
        NSFrameRectWithWidth (NSInsetRect ([self bounds], 1, 1), 2);
    }
}

//----------------------------------------------------------------------

- (void) mouseDown:(NSEvent *)theEvent
{
    unsigned int modifierFlags;
    int clickCount;

    modifierFlags = [theEvent modifierFlags];
    clickCount = [theEvent clickCount];
    if (clickCount == 1)
    {
        [nonretainedDelegate tileView:self singleClicked:theEvent];
    }
    else if (clickCount == 2)
    {
        [nonretainedDelegate tileView:self doubleClicked:theEvent];
#if 0
        BOOL hideOthers;

        hideOthers = (modifierFlags & NSCommandKeyMask) ? YES : NO;
        [nonretainedDelegate tileView:self shouldLaunchApplicationAndHideOthers:hideOthers];
#endif
    }
    else
    {
        [super mouseDown:theEvent];
    }
}

//----------------------------------------------------------------------

- (NSImage *) image
{
    return image;
}

//----------------------------------------------------------------------

- (void) setImage:(NSImage *)newImage
{
    if (newImage == image)
        return;

    SNRelease (image);
    image = [newImage retain];

    [self _calculateCompositePoint];
    [self setNeedsDisplay:YES];
}

//----------------------------------------------------------------------

- (void) _calculateCompositePoint
{
    if (image != nil)
    {
        NSSize imageSize;

        imageSize = [image size];

        // Don't scale smaller images, but be sure to shrink larger ones.
        // XXX: Should we worry about proportionality?
        if (imageSize.height > 48 || imageSize.width > 48)
        {
            [image setScalesWhenResized:YES];
            [image setSize:NSMakeSize (48, 48)];
            compositePoint = NSMakePoint (8, 8);
        }
        else
        {
            compositePoint = NSMakePoint ((64 - imageSize.width) / 2, (64 - imageSize.height) / 2);
        }
    }
}

//----------------------------------------------------------------------

- datasource
{
    return nonretainedDatasource;
}

//----------------------------------------------------------------------

- (void) setDatasource:newDatasource
{
    if (newDatasource == nonretainedDatasource)
        return;

    nonretainedDatasource = newDatasource;
    [self setNeedsDisplay:YES];
}

//----------------------------------------------------------------------

- delegate
{
    return nonretainedDelegate;
}

//----------------------------------------------------------------------

- (void) setDelegate:newDelegate
{
    nonretainedDelegate = newDelegate;
}

//======================================================================
// Covers for the datasource methods
//======================================================================

- (NSImage *) animationImage
{
    return [nonretainedDatasource animationImage];
}

//----------------------------------------------------------------------

- (BOOL) shouldAutoLaunch
{
    return (nonretainedDatasource == nil) ? NO : [nonretainedDatasource shouldAutoLaunch];
}

//----------------------------------------------------------------------

- (BOOL) shouldAnimate
{
    return (nonretainedDatasource == nil) ? NO : [nonretainedDatasource shouldAnimate];
}

//----------------------------------------------------------------------

- (BOOL) isPersistent
{
    return (nonretainedDatasource == nil) ? NO : [nonretainedDatasource isPersistent];
}

//----------------------------------------------------------------------

- (BOOL) isRunning
{
    return (nonretainedDatasource == nil) ? NO : [nonretainedDatasource isRunning];
}

//----------------------------------------------------------------------

- (BOOL) isGhosted
{
    return (nonretainedDatasource == nil) ? NO : [nonretainedDatasource isGhosted];
}

//----------------------------------------------------------------------

- (BOOL) isSelected
{
    return (nonretainedDatasource == nil) ? NO : [nonretainedDatasource isSelected];
}

//======================================================================
// NSDraggingDestination
//======================================================================

- (BOOL) prepareForDragOperation:(id <NSDraggingInfo>)sender
{
    return YES;
}

//----------------------------------------------------------------------

- (BOOL) performDragOperation:(id <NSDraggingInfo>)sender
{
    return YES;
}

//----------------------------------------------------------------------

- (void) concludeDragOperation:(id <NSDraggingInfo>)sender
{
    NSPasteboard *draggingPasteboard;
    NSArray *filenames;
    
    draggingPasteboard = [sender draggingPasteboard];
    //NSLog (@"draggingPasteboard: %@", draggingPasteboard);

    //NSLog (@"available types: %@", [draggingPasteboard types]);

    if ([draggingPasteboard availableTypeFromArray:[NSArray arrayWithObject:NSFilenamesPboardType]] != nil)
    {
        filenames = [draggingPasteboard propertyListForType:NSFilenamesPboardType];
        //NSLog (@"[%@] filenames: %@", NSStringFromClass ([filenames class]), filenames);

        [nonretainedDelegate tileView:self acceptedFiles:filenames];
    }
}

//----------------------------------------------------------------------

- (unsigned int) draggingEntered:(id <NSDraggingInfo>)sender
{
    NSPasteboard *pasteboard;
    unsigned int sourceDragMask;

    sourceDragMask = [sender draggingSourceOperationMask];
    pasteboard = [sender draggingPasteboard];
    if ([[pasteboard types] indexOfObject:NSFilenamesPboardType] != NSNotFound)
    {
        if (sourceDragMask == NSDragOperationGeneric)
            return NSDragOperationGeneric;
    }

    return NSDragOperationNone;
}

//----------------------------------------------------------------------

- (unsigned int) draggingUpdated:(id <NSDraggingInfo>)sender
{
    NSPasteboard *pasteboard;
    unsigned int sourceDragMask;

    sourceDragMask = [sender draggingSourceOperationMask];
    pasteboard = [sender draggingPasteboard];
    if ([[pasteboard types] indexOfObject:NSFilenamesPboardType] != NSNotFound)
    {
        if (sourceDragMask == NSDragOperationGeneric)
            return NSDragOperationGeneric;
    }

    return NSDragOperationNone;
}

//----------------------------------------------------------------------

- (void) draggingExited:(id <NSDraggingInfo>)sender
{
}

//======================================================================
// Other
//======================================================================

- (void) styleDidChange:(NSNotification *)aNotification
{
    [self setNeedsDisplay:YES];
}

@end
