#import "Fiend.h"

RCS_ID ("$Id$");

#import "FITileWindowClone.h"

#import "Fiendish.h"

@implementation FITileWindowClone

static NSImage *_crossbonesImage = nil;

+ (void) initialize
{
    static BOOL initialized = NO;

    [super initialize];
    if (initialized == YES)
        return;
    initialized = YES;

    _crossbonesImage = [[NSImage imageNamed:@"kill" inBundleForClass:self] retain];
    OBPOSTCONDITION (_crossbonesImage != nil);
}

- initWithWindow:(NSWindow *)originalWindow isGhosted:(BOOL)isGhosted
{
    NSImage *cvImage;
    NSRect originalFrame, imageViewFrame;
    NSSize thisSize;
    NSView *sourceContentView;

    int sourceWindowNumber, status;

    if ([super init] == nil)
        return nil;

    originalFrame = [originalWindow frame];
    thisSize = [self frame].size;
    OBASSERT (NSEqualSizes (originalFrame.size, thisSize) == YES);

    [self setLevel:[originalWindow level]];
    [self setFrameOrigin:originalFrame.origin];

    sourceContentView = [originalWindow contentView];

    imageViewFrame = [sourceContentView frame];
    mainImageView = [[NSImageView allocWithZone:[self zone]] initWithFrame:imageViewFrame];
    [self setContentView:mainImageView];

    NSConvertWindowNumberToGlobal ([originalWindow windowNumber], &sourceWindowNumber);

    originalFrame.origin = NSZeroPoint;
    cvImage = [[NSImage allocWithZone:[self zone]] initWithSize:originalFrame.size];
    [cvImage lockFocus];
    // Just having the content view drawRect: doesn't recursively have subviews draw.
    status = safeCopyWindow (originalFrame.origin.x, originalFrame.origin.y,
                             originalFrame.size.width, originalFrame.size.height,
                             sourceWindowNumber);

    if (isGhosted == YES)
    {
        NSRect ghostRect;

        ghostRect = NSInsetRect (originalFrame, 2, 2);
        [[NSColor colorWithCalibratedWhite:1.0 alpha:0.666] set];
        PScompositerect (ghostRect.origin.x, ghostRect.origin.y, ghostRect.size.width, ghostRect.size.height,
                         NSCompositeSourceOver);
    }
    [cvImage unlockFocus];

    if (status != 0)
    {
        NSLog (@"safeCopyWindow() failed.");
    }

    [mainImageView setImage:cvImage];
    [cvImage release];

    imageViewFrame.origin = NSZeroPoint;
    secondImageView = [[NSImageView allocWithZone:[self zone]] initWithFrame:imageViewFrame];
    [secondImageView setImage:_crossbonesImage];

    showCrossbones = NO;

    return self;
}

//----------------------------------------------------------------------

- (void) dealloc
{
    SNRelease (mainImageView);
    SNRelease (secondImageView);

    [super dealloc];
}

//----------------------------------------------------------------------

- (BOOL) showCrossbones
{
    return showCrossbones;
}

//----------------------------------------------------------------------

- (void) setShowCrossbones:(BOOL)flag
{
    if (flag == showCrossbones)
        return;

    showCrossbones = flag;

    if (showCrossbones == YES)
        [mainImageView addSubview:secondImageView];
    else
        [secondImageView removeFromSuperview];
}

@end
