//
// $Id$
//

#import <AppKit/AppKit.h>

#import "FIDockViewDelegateProtocol.h"
#import "FIDockViewDatasourceProtocol.h"
#import "FITileDelegateProtocol.h"

extern NSString *FIApplicationPboardType;

@class FIAppInfo;
@class FIDock;
@class FIDockView;
@class FILevel;
@class FILevelRepresentation;
@class FITile;
@class FITileWindow;
@class FITileWindowClone;

@interface FIWindowController : NSWindowController <FIDockViewDelegate, FITileDelegate>
{
    FITileWindow *dockWindow;
    FIDockView *dockView;

    FILevelRepresentation *persistentLevelRepresentation;
    NSMutableArray *levelRepresentations;

    // XXX: Should the current level be saved as part of the document?
    FILevelRepresentation *currentLevelRepresentation;

    NSTimer *saveTimer;
    NSMutableArray *selectedTiles;

    // Move into an "undocked" level in model?  It wouldn't be saved as part of document.
    NSMutableArray *undockedApplications;
    NSMutableArray *undockedTiles;

    FIAppInfo *workspaceAppInfo;
}

- init;
- (void) dealloc;

//======================================================================
//" Controlling general attributes
//======================================================================

- (void) setDocument:(NSDocument *)newDocument;
- (NSUndoManager *) undoManager;
- (FIDock *) dock;
- (NSWindow *) dockWindow;

- (BOOL) isDockVisible;
- (void) setIsDockVisible:(BOOL)flag;

- (BOOL) isDockLocked;
- (void) setIsDockLocked:(BOOL)flag;

- (BOOL) isDockSunken;
- (void) setIsDockSunken:(BOOL)flag;

//======================================================================
//" FIDockViewDatasource protocol
//======================================================================

- (NSString *) currentLevelNumber;
- (NSString *) currentLevelName;

//======================================================================
//" FIDockViewDelegate protocol
//======================================================================

- (void) dockView:(FIDockView *)dockView startedDragging:(NSEvent *)theEvent;
- (void) dockView:(FIDockView *)dockView acceptedDraggedApplications:(NSArray *)draggedApplications;
- (void) previousLevel:sender;
- (void) nextLevel:sender;
- (void) activateApplication:sender;
- (void) toggleDockVisible:sender;
- (void) toggleDockSunken:sender;
- (void) toggleDockLocked:sender;
- (void) activateWorkspace:sender;

//======================================================================
//" Controlling tile window levels
//======================================================================

- (int) tileWindowLevel;
- (void) resetWindowLevels;

//======================================================================
//" Handling Workspace notifications
//======================================================================

- (void) applicationWillLaunch:(NSNotification *)aNotification;
- (void) applicationDidLaunch:(NSNotification *)aNotification;
- (void) applicationDidTerminate:(NSNotification *)aNotification;

//======================================================================
//" Menu Actions
//======================================================================

- (void) addWorkspaceIcon:sender;
- (void) rotateLevel:sender;
- (void) addLevel:sender;
- (void) clearLevel:sender;
- (void) deleteLevel:sender;

- (void) updateIcons:sender;

- (void) cut:sender;
- (void) copy:sender;
- (void) paste:sender;
- (void) clear:sender;
- (void) selectAll:sender;

//======================================================================
//" FITileDelegate protocol
//======================================================================

- (void) repositionTile:(FITile *)tile;
//- (void) selectTile:(FITile *)tile;
//- (void) toggleSelectionOfTile:(FITile *)tile;
- (void) tile:(FITile *)tile draggedGhostWindow:(FITileWindowClone *)ghostWindow toLocation:(DockLocation)targetLocation;

//======================================================================
//" Controlling selected tiles
//======================================================================

- (void) _selectTile:(FITile *)tile;
- (void) _deselectTile:(FITile *)tile;

- (void) selectTile:(FITile *)tile;
- (void) deselectTile:(FITile *)tile;
- (void) toggleSelectionOfTile:(FITile *)tile;
- (void) deselectAllTiles;

//======================================================================
//" Controlling level representations
//======================================================================

- (void) createLevelRepresentations;

- (FILevelRepresentation *) currentLevelRepresentation;
- (void) setCurrentLevelRepresentation:(FILevelRepresentation *)newLevelRepresentation;

- (FILevelRepresentation *) representationForLevel:(FILevel *)level;

//======================================================================
//" Controlling the inspector
//======================================================================

- (void) updateInspector;
- inspectedObject;

//======================================================================
//" Handling model notifications
//======================================================================

- (void) dockDidMoveApplicationBetweenLevels:(NSNotification *)aNotification;
- (void) dockDidAddLevel:(NSNotification *)aNotification;
- (void) dockDidRemoveLevel:(NSNotification *)aNotification;

//======================================================================
//" Controlling tile positions
//======================================================================

- (NSPoint) initialDockTilePoint;

- (void) repositionTilesRelativeToPoint:(NSPoint)newOrigin;
- (void) repositionVisibleTilesRelativeToPoint:(NSPoint)newOrigin;

//======================================================================
//" Handling startup
//======================================================================

- (void) checkForActiveApplications:sender;
- (void) performAutoLaunch:sender;
- (void) delayedLoadFileIcons:sender;

//======================================================================
//" Controlling dock locations
//======================================================================

- (DockLocation) nextAvailableDockLocation;
- (DockLocation) nextAvailableDockLocationOrLocation:(DockLocation)aLocation;

//======================================================================
//" Controlling tiles
//======================================================================

- (void) hideDock;
- (void) unhideDock;
- (void) adjustVisibleTileWindowLevels;

- (FITile *) tileAtDockLocation:(DockLocation)location;
- (void) togglePersistenceOfTile:(FITile *)tile;
- (NSArray *) applicationsForTiles:(NSArray *)tiles;

//======================================================================
//" Controlling applications
//======================================================================

- (void) addApplicationsFromArray:(NSArray *)applications toLevel:(FILevel *)level;
- (void) addApplicationsWithPaths:(NSArray *)applications toLevel:(FILevel *)level;

//======================================================================
//" Other
//======================================================================

- (BOOL) validateMenuItem:(NSMenuItem *)menuItem;

- (void) findNext:sender;
- (void) findPrevious:sender;

@end
