#import "Fiend.h"

RCS_ID ("$Id$");

#import "FIWindowController.h"

#import "FIAppInfo.h"
#import "FIApplication.h"
#import "FIDock.h"
#import "FIDockView.h"
#import "FIDocument.h"
#import "FIEmptyTileView.h"
#import "FIInspectorManager.h"
#import "FILevel.h"
#import "FILevelRepresentation.h"
#import "FISoundManager.h"
#import "FITile.h"
#import "FITileWindow.h"
#import "FITileWindowClone.h"

DEFINE_NSSTRING (FIApplicationPboardType);

@implementation FIWindowController

- init
{
    NSRect contentFrame;
    NSWorkspace *sharedWorkspace;
    NSNotificationCenter *workspaceCenter, *defaultCenter;
    
    dockWindow = [[FITileWindow allocWithZone:[self zone]] init];

    if ([super initWithWindow:dockWindow] == nil)
    {
        [dockWindow release];
        return nil;
    }

    [dockWindow setLevel:[self tileWindowLevel]];
    [dockWindow setFrameOrigin:[self initialDockTilePoint]];
    [dockWindow setFrameAutosaveName:[NSString stringWithFormat:@"DockWindow.%@", [[NSScreen mainScreen] screenSizeString]]];

    contentFrame = [[dockWindow contentView] frame];
    //SNLogRect (contentFrame);

    dockView = [[FIDockView allocWithZone:[self zone]] initWithFrame:contentFrame];
    [dockView setBoundsSize:NSMakeSize (64, 64)];

    persistentLevelRepresentation = nil;
    levelRepresentations = [[NSMutableArray allocWithZone:[self zone]] initWithCapacity:4];

    currentLevelRepresentation = nil;

    saveTimer = nil;
    selectedTiles = [[NSMutableArray allocWithZone:[self zone]] initWithCapacity:8];

    undockedApplications = [[NSMutableArray allocWithZone:[self zone]] initWithCapacity:8];
    undockedTiles = [[NSMutableArray allocWithZone:[self zone]] initWithCapacity:8];
    workspaceAppInfo = nil;

    // Do these last.
    [dockView setDatasource:self];
    [dockView setDelegate:self];
    [dockView setNeedsDisplay:YES];
    [dockWindow setContentView:dockView];

    sharedWorkspace = [NSWorkspace sharedWorkspace];
    workspaceCenter = [NSNotificationCenter fixedWorkspaceCenter];

    [workspaceCenter addObserver:self
                     selector:@selector (applicationWillLaunch:)
                     name:NSWorkspaceWillLaunchApplicationNotification
                     object:sharedWorkspace];

    [workspaceCenter addObserver:self
                     selector:@selector (applicationDidLaunch:)
                     name:NSWorkspaceDidLaunchApplicationNotification
                     object:sharedWorkspace];

    [workspaceCenter addObserver:self
                     selector:@selector (applicationDidTerminate:)
                     name:NSWorkspaceDidTerminateApplicationNotification
                     object:sharedWorkspace];

    defaultCenter = [NSNotificationCenter defaultCenter];

    return self;
}

//----------------------------------------------------------------------

- (void) dealloc
{
    [[NSNotificationCenter fixedWorkspaceCenter] removeObserver:self];
    [[NSNotificationCenter defaultCenter] removeObserver:self];

    SNRelease (workspaceAppInfo);
    SNRelease (undockedTiles);
    SNRelease (undockedApplications);
    SNRelease (selectedTiles);
    SNRelease (saveTimer);
    SNRelease (currentLevelRepresentation);
    SNRelease (levelRepresentations);
    SNRelease (persistentLevelRepresentation);
    SNRelease (dockView);
    SNRelease (dockWindow);
    
    [super dealloc];
}

//======================================================================
// Controlling general attributes
//======================================================================

- (void) setDocument:(NSDocument *)newDocument
{
    NSNotificationCenter *defaultCenter;
    FIDock *dock;
    
    [super setDocument:newDocument];

    SNRelease (currentLevelRepresentation);
    [levelRepresentations removeAllObjects];
    SNRelease (persistentLevelRepresentation);
    
    [self createLevelRepresentations];

    defaultCenter = [NSNotificationCenter defaultCenter];
    [defaultCenter removeObserver:self name:FIDockDidMoveApplicationBetweenLevelsNotification object:nil];
    [defaultCenter removeObserver:self name:FIDockDidAddLevelNotification object:nil];
    [defaultCenter removeObserver:self name:FIDockDidRemoveLevelNotification object:nil];

    dock = [[self document] model];
    if (dock != nil)
    {
        [defaultCenter addObserver:self
                       selector:@selector (dockDidMoveApplicationBetweenLevels:)
                       name:FIDockDidMoveApplicationBetweenLevelsNotification
                       object:dock];

        [defaultCenter addObserver:self
                       selector:@selector (dockDidAddLevel:)
                       name:FIDockDidAddLevelNotification
                       object:dock];

        [defaultCenter addObserver:self
                       selector:@selector (dockDidRemoveLevel:)
                       name:FIDockDidRemoveLevelNotification
                       object:dock];
    }

    // XXX: Everything should be listening for notifications first..
    [self checkForActiveApplications:self];
    if ([self isDockVisible] == YES)
        [self unhideDock];
    [self updateInspector];
}

//----------------------------------------------------------------------

- (NSUndoManager *) undoManager
{
    return [[self document] undoManager];
}

//----------------------------------------------------------------------

- (FIDock *) dock
{
    return [(FIDocument *)[self document] model];
}

//----------------------------------------------------------------------

- (NSWindow *) dockWindow
{
    return dockWindow;
}

//----------------------------------------------------------------------

- (BOOL) isDockVisible
{
    //return [[OFUserDefaults sharedUserDefaults] boolForKey:DK_IsDockVisible];
    return [[NSUserDefaults standardUserDefaults] boolForKey:DK_IsDockVisible];
}

//----------------------------------------------------------------------

- (void) setIsDockVisible:(BOOL)flag
{
    //[[OFUserDefaults sharedUserDefaults] setBool:flag forKey:DK_IsDockVisible];
    [[NSUserDefaults standardUserDefaults] setBool:flag forKey:DK_IsDockVisible];

    [dockView setNeedsDisplay:YES];
}

//----------------------------------------------------------------------

- (BOOL) isDockLocked
{
    return [[OFUserDefaults sharedUserDefaults] boolForKey:DK_DockLocked];
}

//----------------------------------------------------------------------

- (void) setIsDockLocked:(BOOL)flag
{
    [[OFUserDefaults sharedUserDefaults] setBool:flag forKey:DK_DockLocked];
    //[self updateDockView];
    [dockView setNeedsDisplay:YES];
}

//----------------------------------------------------------------------

- (BOOL) isDockSunken
{
    return [[OFUserDefaults sharedUserDefaults] boolForKey:DK_DockSunken];
}

//----------------------------------------------------------------------

- (void) setIsDockSunken:(BOOL)flag
{
    [[OFUserDefaults sharedUserDefaults] setBool:flag forKey:DK_DockSunken];
    [self adjustVisibleTileWindowLevels];
}

//======================================================================
// FIDockViewDatasource protocol
//======================================================================

- (NSString *) currentLevelNumber
{
    int currentIndex;

    currentIndex = [[self dock] indexOfLevel:[currentLevelRepresentation level]];

    return (currentIndex == NSNotFound) ? @"" : [NSString stringWithFormat:@"%d", currentIndex + 1];
}

//----------------------------------------------------------------------

- (NSString *) currentLevelName
{
    return [[self dock] displayNameOfLevel:[currentLevelRepresentation level]];
}

//======================================================================
// FIDockViewDelegate protocol
//======================================================================

- (void) dockView:(FIDockView *)aDockView startedDragging:(NSEvent *)theEvent
{
    NSPoint originalWindowOrigin, originalScreenPoint;
    NSPoint mouseLocation;
    NSPoint newWindowOrigin;
    unsigned int modifierFlags;

    modifierFlags = [theEvent modifierFlags];
    originalWindowOrigin = [dockWindow frame].origin;
    originalScreenPoint = [dockWindow convertBaseToScreen:[theEvent locationInWindow]];

    theEvent = [dockWindow nextEventMatchingMask:(NSLeftMouseUpMask | NSLeftMouseDraggedMask)];
    if ([theEvent type] == NSLeftMouseDragged)
    {
        // You can only move the dock if it is not locked, or if you are control-dragging.
        // Yes, the modifier flags from the mouse down event are okay to use here.
        if ([self isDockLocked] == NO || (modifierFlags & NSControlKeyMask))
        {
            FITileWindowClone *cloneWindow;

            cloneWindow = [[FITileWindowClone allocWithZone:[self zone]] initWithWindow:dockWindow isGhosted:NO];
            [cloneWindow orderFront:self];
            [dockWindow orderOut:self];

            while ([theEvent type] != NSLeftMouseUp)
            {
                mouseLocation = [dockWindow convertBaseToScreen:[theEvent locationInWindow]];
                newWindowOrigin.x = originalWindowOrigin.x + mouseLocation.x - originalScreenPoint.x;
                newWindowOrigin.y = originalWindowOrigin.y + mouseLocation.y - originalScreenPoint.y;

                [cloneWindow setFrameOrigin:newWindowOrigin];
                [self repositionVisibleTilesRelativeToPoint:newWindowOrigin];
                //[nonretainedDelegate dockWindowDidDrag:newWindowOrigin]; // XXX:

                theEvent = [dockWindow nextEventMatchingMask:(NSLeftMouseUpMask | NSLeftMouseDraggedMask)];
            }

            [dockWindow setFrameOrigin:[cloneWindow frame].origin];
            [dockWindow orderFront:self];
            [cloneWindow orderOut:self];
            [cloneWindow release];

            // And reposition the hidden tiles.
            [self repositionTilesRelativeToPoint:newWindowOrigin];
        }
        else
        {
            NSBeep ();

            // And gobble events up to and including the mouse up.
            theEvent = [dockWindow nextEventMatchingMask:NSLeftMouseUpMask];
            [dockWindow discardEventsMatchingMask:NSLeftMouseDraggedMask beforeEvent:theEvent];
        }
    }
    else
    {
        // If we actually dragged, gobble up the mouse up event, otherwise pass it on.
        [aDockView mouseUp:theEvent];
    }
}

//----------------------------------------------------------------------

- (void) dockView:(FIDockView *)dockView acceptedDraggedApplications:(NSArray *)draggedApplications
{
    [self addApplicationsWithPaths:draggedApplications toLevel:[currentLevelRepresentation level]];
    //[[FISoundManager sharedSoundManager] didAddTiles];
}

//----------------------------------------------------------------------

- (void) previousLevel:sender
{
    int count, currentIndex;

    count = [levelRepresentations count];
    if (count < 2)
        return;

    currentIndex = [levelRepresentations indexOfObject:currentLevelRepresentation];
    OBASSERT (currentIndex != NSNotFound);
    currentIndex = (currentIndex == 0) ? count - 1 : currentIndex - 1;
    [self setCurrentLevelRepresentation:[levelRepresentations objectAtIndex:currentIndex]];
}

//----------------------------------------------------------------------

- (void) nextLevel:sender
{
    int count, currentIndex;

    count = [levelRepresentations count];
    if (count < 2)
        return;

    currentIndex = [levelRepresentations indexOfObject:currentLevelRepresentation];
    OBASSERT (currentIndex != NSNotFound);
    currentIndex = (currentIndex + 1) % count;
    [self setCurrentLevelRepresentation:[levelRepresentations objectAtIndex:currentIndex]];
}

//----------------------------------------------------------------------

- (void) activateApplication:sender
{
    // We need to do this, otherwise hiding Fiend and then trying to
    // activate it gets Fiend stuck -- The menus won't work until, for
    // example, you go to another level or hide and unhide the dock.

    [NSApp unhide:self];
    //[NSApp unhideWithoutActivation];
}

//----------------------------------------------------------------------

- (void) toggleDockVisible:sender
{
    if ([self isDockVisible] == YES)
        [self hideDock];
    else
        [self unhideDock];
}

//----------------------------------------------------------------------

- (void) toggleDockSunken:sender
{
    [self setIsDockSunken:![self isDockSunken]];
}

//----------------------------------------------------------------------

- (void) toggleDockLocked:sender
{
    [self setIsDockLocked:![self isDockLocked]];
}

//----------------------------------------------------------------------

- (void) activateWorkspace:sender
{
    [workspaceAppInfo activateApplicationAndHideOthers:NO];
}

//======================================================================
// Controlling tile window levels
//======================================================================

- (int) tileWindowLevel
{
    return ([self isDockSunken] == YES) ? (NSNormalWindowLevel - 1) : NSDockWindowLevel;
}

//----------------------------------------------------------------------

- (void) resetWindowLevels
{
}

//======================================================================
// Handling Workspace notifications
//======================================================================

- (void) applicationWillLaunch:(NSNotification *)aNotification
{
    [persistentLevelRepresentation applicationWillLaunch:aNotification];
    [levelRepresentations makeObjectsPerformSelector:@selector (applicationWillLaunch:) withObject:aNotification];
}

//----------------------------------------------------------------------

- (void) applicationDidLaunch:(NSNotification *)aNotification
{
    [persistentLevelRepresentation applicationDidLaunch:aNotification];
    [levelRepresentations makeObjectsPerformSelector:@selector (applicationDidLaunch:) withObject:aNotification];
}

//----------------------------------------------------------------------

- (void) applicationDidTerminate:(NSNotification *)aNotification
{
    [persistentLevelRepresentation applicationDidTerminate:aNotification];
    [levelRepresentations makeObjectsPerformSelector:@selector (applicationDidTerminate:) withObject:aNotification];
}

//======================================================================
// Menu Actions
//======================================================================

- (void) addWorkspaceIcon:sender
{
    NSString *workspacePath;

    workspacePath = [[OFUserDefaults sharedUserDefaults] objectForKey:DK_WorkspacePath];
    [self addApplicationsWithPaths:[NSArray arrayWithObject:workspacePath] toLevel:[currentLevelRepresentation level]];
}

//----------------------------------------------------------------------

- (void) rotateLevel:sender
{
    [[currentLevelRepresentation level] rotateApplicationsClockwise:YES];
    [[self undoManager] setActionName:@"Rotate Level"];
}

//----------------------------------------------------------------------

- (void) addLevel:sender
{
    FILevel *newLevel;
    FIDock *dock;

    dock = [self dock];
    newLevel = [[FILevel allocWithZone:[dock zone]] init];
    [dock addLevel:newLevel];
    // XXX: This should trigger a notification
    [self setCurrentLevelRepresentation:[self representationForLevel:newLevel]];
    [[self undoManager] setActionName:@"Add Level"];
}

//----------------------------------------------------------------------

- (void) clearLevel:sender
{
    int applicationCount;
    int returnValue;

    applicationCount = [[[currentLevelRepresentation level] applications] count];
    if (applicationCount > 0)
    {
        returnValue = NSRunAlertPanel (@"Clear Level",
                                       @"You are about to remove %d non-persistent application%@ from the current dock level.",
                                       @"Cancel", @"Remove Applications", nil,
                                       applicationCount, applicationCount == 1 ? @"" : @"s");
        if (returnValue == NSAlertAlternateReturn)
        {
            [[currentLevelRepresentation level] clearLevel];
	    //[[FISoundManager sharedSoundManager] didRemoveTilesForDock:[self dock]];
            [[self undoManager] setActionName:@"Clear Level"];
        }
    }
}

//----------------------------------------------------------------------

- (void) deleteLevel:sender
{
    [[self undoManager] setActionName:@"Remove Level"];
}

//----------------------------------------------------------------------

- (void) updateIcons:sender
{
    DSTART;
    [persistentLevelRepresentation updateFileIcons];
    [levelRepresentations makeObjectsPerformSelector:@selector (updateFileIcons)];
    DEND;
}

//----------------------------------------------------------------------

- (void) cut:sender
{
    NSArray *selectedApplications;
    
    [self copy:sender];
    selectedApplications = [self applicationsForTiles:selectedTiles];
    [[self dock] removeApplicationsFromArray:selectedApplications];
    [[self undoManager] setActionName:@"Cut Applications"];
}

//----------------------------------------------------------------------

- (void) copy:sender
{
    NSPasteboard *generalPasteboard;
    NSMutableArray *applications, *applicationPaths;
    FIApplication *application;
    NSEnumerator *tileEnumerator;
    FITile *tile;
    int changeCount;
    NSArray *types;

    // XXX: Add custom application pasteboard type that will preserve the flags...

    generalPasteboard = [NSPasteboard generalPasteboard];
    applications = [NSMutableArray array];
    applicationPaths = [NSMutableArray array];
    tileEnumerator = [selectedTiles objectEnumerator];
    while (tile = [tileEnumerator nextObject])
    {
        application = [tile application];
        [applications addObject:application];
        [applicationPaths addObject:[application path]];
    }
    types = [NSArray arrayWithObjects:FIApplicationPboardType, NSFilenamesPboardType, nil];
    changeCount = [generalPasteboard declareTypes:types owner:nil];
    if ([generalPasteboard setData:[NSArchiver archivedDataWithRootObject:applications] forType:FIApplicationPboardType] == NO)
    {
        NSLog (@"setData:forType: returned NO.");
    }
    if ([generalPasteboard setPropertyList:applicationPaths forType:NSFilenamesPboardType] == NO)
    {
        NSLog (@"setPropertyList:forType: returned NO.");
    }
}

//----------------------------------------------------------------------

- (void) paste:sender
{
    NSPasteboard *generalPasteboard;
    NSString *availableType;
    NSArray *types, *filenames;

    generalPasteboard = [NSPasteboard generalPasteboard];
    types = [NSArray arrayWithObjects:FIApplicationPboardType, NSFilenamesPboardType, nil];
    availableType = [generalPasteboard availableTypeFromArray:types];

    if (availableType == FIApplicationPboardType)
    {
        NSData *data;
        NSArray *applications;

        data = [generalPasteboard dataForType:availableType];
        applications = [NSUnarchiver unarchiveObjectWithData:data];
        [self addApplicationsFromArray:applications toLevel:[currentLevelRepresentation level]];
        [[self undoManager] setActionName:@"Paste Applications"];
    }
    else if (availableType == NSFilenamesPboardType)
    {
        filenames = [generalPasteboard propertyListForType:NSFilenamesPboardType];

        [self addApplicationsWithPaths:filenames toLevel:[currentLevelRepresentation level]];
        [[self undoManager] setActionName:@"Paste Applications"];

        [[FISoundManager sharedSoundManager] didAddTilesForDock:[self dock]];
    }
    else
    {
        NSBeep ();
    }
}

//----------------------------------------------------------------------

- (void) clear:sender
{
    [self deselectAllTiles];
}

//----------------------------------------------------------------------

- (void) selectAll:sender
{
    NSEnumerator *tileEnumerator;
    FITile *tile;

    [self deselectAllTiles];
    tileEnumerator = [[persistentLevelRepresentation tiles] objectEnumerator];
    while (tile = [tileEnumerator nextObject])
        [self _selectTile:tile];

    tileEnumerator = [[currentLevelRepresentation tiles] objectEnumerator];
    while (tile = [tileEnumerator nextObject])
        [self _selectTile:tile];
}

//======================================================================
// FITileDelegate protocol
//======================================================================

- (void) repositionTile:(FITile *)tile
{
    [tile repositionRelativeToPoint:[dockWindow frame].origin];
}

//----------------------------------------------------------------------

- (void) tile:(FITile *)aTile draggedGhostWindow:(FITileWindowClone *)ghostWindow toLocation:(DockLocation)targetLocation
{
    NSRect dockTileFrame;
    NSPoint tileOrigin;
    FITile *tile;
    DockLocation tileLocation;

    tileLocation = [[aTile application] location];

    tile = [self tileAtDockLocation:targetLocation];
    if (tile != nil && tile != aTile)
    {
        targetLocation = [self nextAvailableDockLocationOrLocation:tileLocation];
    }

    dockTileFrame = [dockWindow frame];
    tileOrigin = dockTileFrame.origin;
    tileOrigin.x += dockTileFrame.size.width * targetLocation.x;
    tileOrigin.y += dockTileFrame.size.height * targetLocation.y;
    [ghostWindow setFrameOrigin:tileOrigin];
}

//======================================================================
// Controlling selected tiles
//======================================================================

- (void) _selectTile:(FITile *)tile
{
    if ([tile isSelected] == NO)
    {
        [tile setIsSelected:YES];
        [selectedTiles addObject:tile];
    }
}

//----------------------------------------------------------------------

- (void) _deselectTile:(FITile *)tile
{
    if ([tile isSelected] == YES)
    {
        [tile setIsSelected:NO];
        [selectedTiles removeObject:tile];
    }
}

//----------------------------------------------------------------------

- (void) selectTile:(FITile *)tile
{
    [self deselectAllTiles];
    [self _selectTile:tile];
    [self updateInspector];
}

//----------------------------------------------------------------------

- (void) deselectTile:(FITile *)tile
{
    [self _deselectTile:tile];
    [self updateInspector];
}

//----------------------------------------------------------------------

- (void) toggleSelectionOfTile:(FITile *)tile
{
    if ([tile isSelected] == YES)
        [self _deselectTile:tile];
    else
        [self _selectTile:tile];

    [self updateInspector];
}

//----------------------------------------------------------------------

- (void) deselectAllTiles
{
    NSEnumerator *tileEnumerator;
    FITile *tile;

    tileEnumerator = [selectedTiles objectEnumerator];
    while (tile = [tileEnumerator nextObject])
    {
        [tile setIsSelected:NO];
    }

    [selectedTiles removeAllObjects];
    [self updateInspector];
}

//======================================================================
// Controlling level representations
//======================================================================

- (void) createLevelRepresentations
{
    FIDock *dock;
    NSEnumerator *levelEnumerator;
    FILevel *level;
    FILevelRepresentation *levelRepresentation;

    dock = [self dock];
    OBASSERT (dock != nil);

    persistentLevelRepresentation = [[FILevelRepresentation allocWithZone:[self zone]] initWithLevel:[dock persistentLevel]];
    [persistentLevelRepresentation setWindowController:self];
    levelEnumerator = [[dock levels] objectEnumerator];
    while (level = [levelEnumerator nextObject])
    {
        levelRepresentation = [[FILevelRepresentation allocWithZone:[self zone]] initWithLevel:level];
        [levelRepresentation setWindowController:self];
        [levelRepresentations addObject:levelRepresentation];
    }

    OBASSERT ([levelRepresentations count] > 0);
    currentLevelRepresentation = [[levelRepresentations objectAtIndex:0] retain];
    [dockView setNeedsDisplay:YES];
    [dockWindow orderFront:self];

    //SNLog (persistentLevelRepresentation);
    //SNLog (levelRepresentations);

    // XXX: And then adjust the tile origins relative to the dock tile.
    [self repositionTilesRelativeToPoint:[dockWindow frame].origin];
}

//----------------------------------------------------------------------

- (FILevelRepresentation *) currentLevelRepresentation
{
    return currentLevelRepresentation;
}

//----------------------------------------------------------------------

- (void) setCurrentLevelRepresentation:(FILevelRepresentation *)newLevelRepresentation
{
    if (newLevelRepresentation == currentLevelRepresentation)
        return;

    OBASSERT ([levelRepresentations containsObject:newLevelRepresentation]);

    //[currentLevelRepresentation hideLevel];
    [currentLevelRepresentation setIsVisible:NO];

    SNRelease (currentLevelRepresentation);
    currentLevelRepresentation = [newLevelRepresentation retain];

    [dockView display];
    [self unhideDock];
}

//----------------------------------------------------------------------

- (FILevelRepresentation *) representationForLevel:(FILevel *)level
{
    FILevelRepresentation *targetRepresentation = nil;

    if (level == [persistentLevelRepresentation level])
    {
        targetRepresentation = persistentLevelRepresentation;
    }
    else
    {
        NSEnumerator *levelRepresentationEnumerator;
        FILevelRepresentation *levelRepresentation;

        levelRepresentationEnumerator = [levelRepresentations objectEnumerator];
        while (levelRepresentation = [levelRepresentationEnumerator nextObject])
        {
            if (level == [levelRepresentation level])
            {
                targetRepresentation = levelRepresentation;
                break;
            }
        }
    }

    return targetRepresentation;
}

//======================================================================
// Controlling the inspector
//======================================================================

- (void) updateInspector
{
    [[FIInspectorManager sharedInspectorManager] setInspectedObject:[self inspectedObject]];
}

//----------------------------------------------------------------------

- inspectedObject
{
    if ([selectedTiles count] > 0)
        return [selectedTiles objectAtIndex:0];

    return self;
    //return currentLevelRepresentation;
}

//======================================================================
// Handling model notifications
//======================================================================

- (void) dockDidMoveApplicationBetweenLevels:(NSNotification *)aNotification
{
    NSDictionary *userInfo;
    FIApplication *application;
    FILevel *oldLevel, *newLevel;
    FILevelRepresentation *oldLevelRepresentation, *newLevelRepresentation;
    FITile *tile;
    
    userInfo = [aNotification userInfo];
    application = [userInfo objectForKey:@"application"];
    OBASSERT (application != nil);
#if 1
    // XXX: Since we don't have a back pointer from the tile to the level representation.
    oldLevel = [userInfo objectForKey:@"oldLevel"];
    OBASSERT (oldLevel != nil);
    oldLevelRepresentation = [self representationForLevel:oldLevel];
    OBASSERT (oldLevelRepresentation != nil);
#else
    // - get tile
    // - get level rep (this is the old one)
    oldLevel = [application level];
    OBASSERT (oldLevel != nil);
    oldLevelRepresentation = [self representationForLevel:oldLevel];
    OBASSERT (oldLevelRepresentation != nil);
#endif
    
    newLevel = [application level];
    newLevelRepresentation = [self representationForLevel:newLevel];
    OBASSERT (newLevelRepresentation != nil);

    OBASSERT (oldLevelRepresentation != newLevelRepresentation); // This is bad, it crashes!

    tile = [oldLevelRepresentation tileForApplication:application];
    OBASSERT (tile != nil);
    
    [tile retain];
    [oldLevelRepresentation _removeTile:tile];
    [newLevelRepresentation addTile:tile checkActive:NO];
    [tile release];
}

//----------------------------------------------------------------------

// XXX: We need to worry about the index of the level.

- (void) dockDidAddLevel:(NSNotification *)aNotification
{
    NSDictionary *userInfo;
    FILevel *newLevel;
    int targetIndex;
    FILevelRepresentation *newLevelRepresentation;
    
    userInfo = [aNotification userInfo];
    newLevel = [userInfo objectForKey:@"level"];
    OBPOSTCONDITION (newLevel != nil);

    targetIndex = [[[self dock] levels] indexOfObject:newLevel];
    OBPOSTCONDITION (targetIndex != NSNotFound);

    newLevelRepresentation = [[FILevelRepresentation allocWithZone:[self zone]] initWithLevel:newLevel];
    [newLevelRepresentation setWindowController:self];
    [levelRepresentations insertObject:newLevelRepresentation atIndex:targetIndex];

    [dockView setNeedsDisplay:YES];
}

//----------------------------------------------------------------------

- (void) dockDidRemoveLevel:(NSNotification *)aNotification
{
    NSDictionary *userInfo;
    FILevel *oldLevel;
    FILevelRepresentation *oldLevelRepresentation;
    int oldIndex;
    
    userInfo = [aNotification userInfo];
    oldLevel = [userInfo objectForKey:@"level"];
    OBPOSTCONDITION (oldLevel != nil);

    oldLevelRepresentation = [self representationForLevel:oldLevel];

    oldIndex = [levelRepresentations indexOfObject:oldLevelRepresentation];
    [levelRepresentations removeObject:oldLevelRepresentation];
    if (oldLevelRepresentation == currentLevelRepresentation)
    {
        int count;
        count = [levelRepresentations count];
        OBPOSTCONDITION (oldIndex != NSNotFound);
        if (oldIndex >= count)
            oldIndex = count - 1;
        [self setCurrentLevelRepresentation:[levelRepresentations objectAtIndex:oldIndex]];
    }

    [dockView setNeedsDisplay:YES];
}

//======================================================================
// Controlling tile positions
//======================================================================

//----------------------------------------------------------------------
//" This is used one time, when you first run Fiend, to place the Fiend
//" tile near the upper-left corner of the main screen.  After that the
//" tile window's frame is auto-saved in the defaults database.
//"
//" I don't know whether it will handle multiple screens.
//----------------------------------------------------------------------

- (NSPoint) initialDockTilePoint
{
    OFUserDefaults *defaults;
    NSRect mainScreenFrame;
    NSPoint fiendTileOrigin;
    float tileWidth;

    defaults = [OFUserDefaults sharedUserDefaults];
    tileWidth = [defaults floatForKey:@"TileSize"];

    mainScreenFrame = [[NSScreen mainScreen] visibleFrame];

    fiendTileOrigin.x = NSMaxX (mainScreenFrame) - tileWidth - 5;
    fiendTileOrigin.y = NSMaxY (mainScreenFrame) - tileWidth - 5;

    return fiendTileOrigin;
}

//----------------------------------------------------------------------

- (void) repositionTilesRelativeToPoint:(NSPoint)newOrigin
{
    NSEnumerator *levelRepresentationEnumerator;
    FILevelRepresentation *levelRepresentation;

    [persistentLevelRepresentation repositionRelativeToPoint:newOrigin];
    levelRepresentationEnumerator = [levelRepresentations objectEnumerator];
    while (levelRepresentation = [levelRepresentationEnumerator nextObject])
    {
        [levelRepresentation repositionRelativeToPoint:newOrigin];
    }
}

//----------------------------------------------------------------------

- (void) repositionVisibleTilesRelativeToPoint:(NSPoint)newOrigin
{
    [persistentLevelRepresentation repositionRelativeToPoint:newOrigin];
    [currentLevelRepresentation repositionRelativeToPoint:newOrigin];
}

//======================================================================
// Handling startup
//======================================================================

- (void) checkForActiveApplications:sender
{
    NSDictionary *activeApplications;
    FIApplication *workspace;
    OFUserDefaults *defaults;

    activeApplications = [FIAppInfo activeAppInfo];
    [persistentLevelRepresentation checkForActiveApplications:activeApplications];
    [levelRepresentations makeObjectsPerformSelector:@selector (checkForActiveApplications:) withObject:activeApplications];

    SNRelease (workspaceAppInfo);
    defaults = [OFUserDefaults sharedUserDefaults];
    workspace = [[FIApplication alloc] initWithRepresentedPath:[defaults objectForKey:DK_WorkspacePath]];
    workspaceAppInfo = [[activeApplications objectForKey:[workspace shortName]] retain];
    [workspace release];
}

//----------------------------------------------------------------------

- (void) performAutoLaunch:sender
{
    [persistentLevelRepresentation performAutoLaunch];
    [levelRepresentations makeObjectsPerformSelector:@selector (performAutoLaunch)];
}

//----------------------------------------------------------------------
//" The file icons for tiles are loaded lazily just before the tile is
//" put on the screen, to speed up the application launch.  This method
//" is called a certain time after launch to load the remaining icons,
//" so that the first switch between levels will be faster.
//----------------------------------------------------------------------

- (void) delayedLoadFileIcons:sender
{
    [persistentLevelRepresentation updateFileIconsIfNeeded];
    [levelRepresentations makeObjectsPerformSelector:@selector (updateFileIconsIfNeeded)];
}

//======================================================================
// Controlling dock locations
//======================================================================

// XXX: Hmm.  Move part of this into FIDock, and then have delegate able to restrict it?

- (DockLocation) nextAvailableDockLocation
{
    DockLocation location;

    location.x = -1000;
    location.y = -1000;
    return [self nextAvailableDockLocationOrLocation:location];
}

//----------------------------------------------------------------------

- (DockLocation) nextAvailableDockLocationOrLocation:(DockLocation)aLocation
{
    int minx, miny, maxx, maxy;
    DockLocation current;
    NSRect mainScreenFrame;
    OFUserDefaults *defaults;
    NSString *fiendTileCorner;
    BOOL placeTilesVertically;
    float tileWidth;
    NSPoint dockTileOrigin;

    int endX, endY, deltaX, deltaY;

    // Calculate number of tiles the will fit below, left, above, and right of the main tile.
    // (Calculate total number of tiles that will fit on screen.) xxx

    defaults = [OFUserDefaults sharedUserDefaults];
    tileWidth = [defaults floatForKey:DK_TileSize];

    mainScreenFrame = [[NSScreen mainScreen] frame];
    dockTileOrigin = [dockWindow frame].origin;

    miny = - (dockTileOrigin.y - mainScreenFrame.origin.y) / tileWidth;
    maxy = (NSMaxY (mainScreenFrame) - (dockTileOrigin.y + tileWidth)) / tileWidth;

    minx = - (dockTileOrigin.x - mainScreenFrame.origin.x) / tileWidth;
    maxx = (NSMaxX (mainScreenFrame) - (dockTileOrigin.x + tileWidth)) / tileWidth;


    fiendTileCorner = [defaults objectForKey:DK_FiendTileCorner];
    placeTilesVertically = [defaults boolForKey:DK_PlaceTilesVertically];

    // Two cases, place either vertically or horizontally first.
    // Then, depending on the corner, we set the start x,y, end x,y, and delta x,y's.
    // The start x,y are always 0,0.

    if ([fiendTileCorner isEqual:@"SW"] == YES)
    {
        endX = maxx;
        endY = maxy;
        deltaX = 1;
        deltaY = 1;
    }
    else if ([fiendTileCorner isEqual:@"NW"] == YES)
    {
        endX = maxx;
        endY = miny;
        deltaX = 1;
        deltaY = -1;
    }
    else if ([fiendTileCorner isEqual:@"SE"] == YES)
    {
        endX = minx;
        endY = maxy;
        deltaX = -1;
        deltaY = 1;
    }
    else // NE
    {
        endX = minx;
        endY = miny;
        deltaX = -1;
        deltaY = -1;
    }

    //NSLog (@"end: (%d,%d), delta: (%d,%d)", endX, endY, deltaX, deltaY);

    // Scan persistent and current dockLevels for empty tile space

    if (placeTilesVertically == YES)
    {
        //NSLog (@"Placing tiles vertically.");
        for (current.x = 0; current.x != (endX + deltaX) ; current.x += deltaX)
        {
            for (current.y = 0; current.y != (endY + deltaY); current.y += deltaY)
            {
                // The (0,0) location is reserved for the dock window.
                if (current.x == 0 && current.y == 0)
                    continue;
                if (current.x == aLocation.x && current.y == aLocation.y)
                {
                    //NSLog (@"Dock Location: (%d,%d)", current.x, current.y);
                    return current;
                }
            
                if ([[persistentLevelRepresentation level] isDockLocationEmpty:current] == YES
                    && [[currentLevelRepresentation level] isDockLocationEmpty:current] == YES)
                {
                    //NSLog (@"Dock Location: (%d,%d)", current.x, current.y);
                    return current;
                }
            }
        }
    }
    else
    {
        //NSLog (@"Placing tiles horizontally.");
        for (current.y = 0; current.y != (endY + deltaY); current.y += deltaY)
        {
            for (current.x = 0; current.x != (endX + deltaX) ; current.x += deltaX)
            {
                // The (0,0) location is reserved for the dock window.
                if (current.x == 0 && current.y == 0)
                    continue;
                if (current.x == aLocation.x && current.y == aLocation.y)
                {
                    //NSLog (@"Dock Location: (%d,%d)", current.x, current.y);
                    return current;
                }
            
                if ([[persistentLevelRepresentation level] isDockLocationEmpty:current] == YES
                    && [[currentLevelRepresentation level] isDockLocationEmpty:current] == YES)
                {
                    //NSLog (@"Dock Location: (%d,%d)", current.x, current.y);
                    return current;
                }
            }
        }
    }

    NSLog (@"No dock locations available.");

    //NSLog (@"Dock Location: (%d,%d)", current.x, current.y);

    // We never want to cover up the main Fiend tile. (Perhaps it should have a higher window level...)
    current.x = deltaX;
    current.y = deltaY;

    return current;
}

//======================================================================
// Controlling tiles
//======================================================================

// XXX: Here, the hide method calls the set method.  Other places (levelRep, tile)
//     have the set method call the hide method...
- (void) hideDock
{
    //[persistentLevelRepresentation hideLevel];
    //[currentLevelRepresentation hideLevel];
    [persistentLevelRepresentation setIsVisible:NO];
    [currentLevelRepresentation setIsVisible:NO];

    [self setIsDockVisible:NO];
}

//----------------------------------------------------------------------

- (void) unhideDock
{
    // There will be a delay between showing the persistent level and showing
    // the main level at startup.  The solution is to update the file icons
    // first, then unhide...
    [self adjustVisibleTileWindowLevels]; // XXX: adjust the window levels immediately when they change?
    //[persistentLevelRepresentation unhideLevel];
    //[currentLevelRepresentation unhideLevel];
    [persistentLevelRepresentation setIsVisible:YES];
    [currentLevelRepresentation setIsVisible:YES];

    [self setIsDockVisible:YES];
}

//----------------------------------------------------------------------

- (void) adjustVisibleTileWindowLevels
{
    [dockWindow setLevel:[self tileWindowLevel]];
    [persistentLevelRepresentation setTileWindowLevel:[self tileWindowLevel]];
    [currentLevelRepresentation setTileWindowLevel:[self tileWindowLevel]];
}

//----------------------------------------------------------------------

// Just deals with persistent & current...
- (FITile *) tileAtDockLocation:(DockLocation)location
{
    FITile *tile;

    tile = [persistentLevelRepresentation tileAtDockLocation:location];
    if (tile == nil)
        tile = [currentLevelRepresentation tileAtDockLocation:location];

    return tile;
}

//----------------------------------------------------------------------

// FIDock doesn't know about "current" level.
- (void) togglePersistenceOfTile:(FITile *)tile
{
    FIApplication *application;
    FIDock *dock;

    application = [tile application];
    dock = [[self document] model];
    if ([application isPersistent] == YES)
    {
        [dock moveApplication:application toLevel:[currentLevelRepresentation level]];
#if 0
        [application retain];
        [[persistentLevelRepresentation level] removeApplication:application];
        [application setIsPersistent:NO];
        [[currentLevelRepresentation level] addApplication:application];
        [application release];
#endif
        [[self undoManager] setActionName:@"Clear Persistent"];
    }
    else
    {
        [dock moveApplication:application toLevel:[persistentLevelRepresentation level]];
#if 0
        FILevel *level;

        level = [application level];
        OBASSERT ([level containsObject:application] == YES);
        [application retain];
        [level removeApplication:application];
        [application setIsPersistent:YES];
        [[persistentLevelRepresentation level] addApplication:application];
        [application release];
#endif
        [[self undoManager] setActionName:@"Set Persistent"];
    }
}

//----------------------------------------------------------------------

// XXX: Is this really a good name/place for this?
- (NSArray *) applicationsForTiles:(NSArray *)tiles
{
    NSMutableArray *applications;
    NSEnumerator *tileEnumerator;
    FITile *tile;

    applications = [NSMutableArray array];
    tileEnumerator = [tiles objectEnumerator];
    while (tile = [tileEnumerator nextObject])
    {
        [applications addObject:[tile application]];
    }

    return applications;
}

//======================================================================
// Controlling applications
//======================================================================

- (void) addApplicationsFromArray:(NSArray *)applications toLevel:(FILevel *)level
{
    NSEnumerator *applicationEnumerator;
    FIApplication *application;
    FIDock *dock;
    BOOL didAddAnApplication = NO;

    dock = [[self document] model];
    applicationEnumerator = [applications objectEnumerator];
    while (application = [applicationEnumerator nextObject])
    {
        if ([dock containsApplicationWithPath:[application path]] == NO)
        {
            [application setIsPersistent:NO]; // We don't want to preserve this flag.
            [level addApplication:application];
            didAddAnApplication = YES;
        }
        else
        {
            NSLog (@"Application is already on the dock: %@", [application path]);
        }
    }

    if (didAddAnApplication == YES)
    {
        // XXX: BUT THIS WON'T WORK WITH UNDO!
        //[level checkForActiveApplications:[FIAppInfo activeAppInfo]];
        [[FISoundManager sharedSoundManager] didAddTilesForDock:[self dock]];
        [[self undoManager] setActionName:@"Add Applications"];
    }
}

//----------------------------------------------------------------------
// XXX: Have this create the applications and call the previous method?  No, the nextDockLocation wouldn't work, then.

- (void) addApplicationsWithPaths:(NSArray *)applications toLevel:(FILevel *)level
{
    NSEnumerator *filenameEnumerator;
    NSString *filename;
    FIApplication *application;
    DockLocation dockLocation;
    NSDictionary *activeApplications;
    FIDock *dock;
    BOOL didAddAnApplication = NO;

    activeApplications = [FIAppInfo activeAppInfo];
    dock = [[self document] model];

    filenameEnumerator = [applications objectEnumerator];
    while (filename = [filenameEnumerator nextObject])
    {
        if ([filename isApplicationPath] == YES)
        {
            if ([dock containsApplicationWithPath:filename] == NO)
            {
                // Get next available icon position (based on current & persistent dockLevels)
                application = [[FIApplication allocWithZone:[self zone]] initWithRepresentedPath:filename];
                dockLocation = [self nextAvailableDockLocation];
                [application setLocation:dockLocation];
                [level addApplication:application];
                [application release];
                didAddAnApplication = YES;
            }
            else
            {
                NSLog (@"Application is already on the dock: %@", filename);
            }
        }
        else
        {
            NSLog (@"File is not an application (.app, .debug, or .profile): %@", filename);
        }
    }

    if (didAddAnApplication == YES)
    {
        [[FISoundManager sharedSoundManager] didAddTilesForDock:[self dock]];
        [[self undoManager] setActionName:@"Add Applications"];
    }
}

//======================================================================
// Other
//======================================================================

- (BOOL) validateMenuItem:(NSMenuItem *)menuItem
{
    SEL action = [menuItem action];
    BOOL valid = NO;

    if (action == @selector (nextLevel:)
        || action == @selector (previousLevel:)
        || action == @selector (deleteLevel:))
    {
        valid = [levelRepresentations count] > 1;
    }
    else if (action == @selector (toggleDockVisible:)
             || action == @selector (toggleDockLocked:)
             || action == @selector (addWorkspaceIcon:)
             || action == @selector (rotateLevel:)
             || action == @selector (addLevel:)
             || action == @selector (selectAll:)
             || action == @selector (clearLevel:)
             || action == @selector (updateIcons:))
    {
        valid = YES;
    }
    else if (action == @selector (cut:)
             || action == @selector (copy:)
             || action == @selector (clear:))
    {
        valid = [selectedTiles count] > 0;
    }
    else if (action == @selector (paste:))
    {
        // Should we check the pasteboard type?
        valid = YES;
    }
    else if (action == @selector (findNext:)
             || action == @selector (findPrevious:))
    {
        valid = YES;
    }

    return valid;
}

//----------------------------------------------------------------------

- (void) findNext:sender
{
    NSPasteboard *findPasteboard;
    NSString *availableType;
    NSString *searchString = nil;
    FIApplication *startingApplication, *nextApplication;
    BOOL ignoreCase;
    OFUserDefaults *defaults;

    findPasteboard = [NSPasteboard pasteboardWithName:NSFindPboard];
    availableType = [findPasteboard availableTypeFromArray:[NSArray arrayWithObject:NSStringPboardType]];
    if (availableType == NSStringPboardType)
    {
        searchString = [findPasteboard stringForType:NSStringPboardType];
    }
    if (searchString == nil)
        searchString = @"";

    defaults = [OFUserDefaults sharedUserDefaults];
    ignoreCase = [defaults boolForKey:DK_FindIgnoresCase];
    if ([defaults boolForKey:DK_FindApplications] == YES)
    {
        if ([selectedTiles count] > 0)
            startingApplication = [[selectedTiles objectAtIndex:0] application];
        else
            startingApplication = nil;

        //NSLog (@"starting at: %@", [startingApplication path]);
        nextApplication = [[self dock] findNextApplication:startingApplication withString:searchString ignoreCase:ignoreCase];
        //SNLog (nextApplication);
        if (nextApplication == nil)
        {
            [self deselectAllTiles];
            NSBeep ();
        }
        else
        {
            FILevelRepresentation *levelRepresentation;
            FITile *tile;
            levelRepresentation = [self representationForLevel:[nextApplication level]];
            OBPOSTCONDITION (levelRepresentation != nil);
            if (levelRepresentation != persistentLevelRepresentation && levelRepresentation != currentLevelRepresentation)
                [self setCurrentLevelRepresentation:levelRepresentation];
        
            tile = [levelRepresentation tileForApplication:nextApplication];
            OBASSERT (tile != nil);
            [self selectTile:tile]; // Is it okay if it's nil?
            // XXX: And switch to appropriate level.
        }
    }
    else
    {
        FILevel *nextLevel;

        nextLevel = [[self dock] findNextLevel:[currentLevelRepresentation level] withString:searchString ignoreCase:ignoreCase];
        SNLog (nextLevel);
        if (nextLevel == nil)
            NSBeep ();
        else
            [self setCurrentLevelRepresentation:[self representationForLevel:nextLevel]];
    }
}

//----------------------------------------------------------------------

- (void) findPrevious:sender
{
    NSPasteboard *findPasteboard;
    NSString *availableType;
    NSString *searchString = nil;
    FIApplication *startingApplication, *previousApplication;
    BOOL ignoreCase;
    OFUserDefaults *defaults;

    findPasteboard = [NSPasteboard pasteboardWithName:NSFindPboard];
    availableType = [findPasteboard availableTypeFromArray:[NSArray arrayWithObject:NSStringPboardType]];
    if (availableType == NSStringPboardType)
    {
        searchString = [findPasteboard stringForType:NSStringPboardType];
    }
    if (searchString == nil)
        searchString = @"";

    defaults = [OFUserDefaults sharedUserDefaults];
    ignoreCase = [defaults boolForKey:DK_FindIgnoresCase];

    if ([defaults boolForKey:DK_FindApplications] == YES)
    {
        if ([selectedTiles count] > 0)
            startingApplication = [[selectedTiles objectAtIndex:0] application];
        else
            startingApplication = nil;

    //NSLog (@"starting at: %@", [startingApplication path]);
        previousApplication = [[self dock] findPreviousApplication:startingApplication withString:searchString ignoreCase:ignoreCase];
        //SNLog (previousApplication);
        if (previousApplication == nil)
        {
            [self deselectAllTiles];
            NSBeep ();
        }
        else
        {
            FILevelRepresentation *levelRepresentation;
            FITile *tile;
            levelRepresentation = [self representationForLevel:[previousApplication level]];
            OBPOSTCONDITION (levelRepresentation != nil);
            if (levelRepresentation != persistentLevelRepresentation && levelRepresentation != currentLevelRepresentation)
                [self setCurrentLevelRepresentation:levelRepresentation];
        
            tile = [levelRepresentation tileForApplication:previousApplication];
            OBASSERT (tile != nil);
            [self selectTile:tile]; // Is it okay if it's nil?
            // XXX: And switch to appropriate level.
        }
    }
    else
    {
        FILevel *previousLevel;

        previousLevel = [[self dock] findNextLevel:[currentLevelRepresentation level]
                                     withString:searchString
                                     ignoreCase:ignoreCase];
        SNLog (previousLevel);
        if (previousLevel == nil)
            NSBeep ();
        else
            [self setCurrentLevelRepresentation:[self representationForLevel:previousLevel]];
    }
}

@end
