#import "Fiend.h"

RCSID ("$Id$");

#import "FITileInspector.h"

#import "FIApplication.h"
#import "FIDock.h"
#import "FILevel.h"
#import "FITile.h"
#import "FIWindowController.h"

@implementation FITileInspector

+ sharedTileInspector
{
    static FITileInspector *instance = nil;

    if (instance == nil)
        instance = [[FITileInspector alloc] init];

    return instance;
}

//----------------------------------------------------------------------

- init
{
    if ([super initWithWindowNibName:@"TileInspector"] == nil)
        return nil;

    return self;
}

//----------------------------------------------------------------------

- (void) dealloc
{
    [[NSNotificationCenter defaultCenter] removeObserver:self];
    [super dealloc];
}

//----------------------------------------------------------------------

- (void) windowDidLoad
{
    [super windowDidLoad];

    [[pathField cell] setWraps:YES];
    // XXX: Doesn't work.
    //[[self window] setHidesOnDeactivate:[[OFUserDefaults sharedUserDefaults] boolForKey:DK_HideInspectorOnDeactivate]];
}

//----------------------------------------------------------------------

- (BOOL) canInspectObject:anObject
{
    return anObject != nil && [anObject isKindOfClass:[FITile class]];
}

//----------------------------------------------------------------------

- (void) updateInspector
{
    FITile *tile;
    FIApplication *application;

    BOOL isFirstResponder;
    id firstResponder = nil;

    tile = (FITile *)[self inspectedObject];
    application = [tile application];

    [imageView setImage:[tile fileIcon]];
    [nameField setStringValue:[[application path] lastPathComponent]];

    [pathField setStringValue:[application path]];
    [autolaunchCell setIntValue:[application shouldAutoLaunch]];
    [animateCell setIntValue:[application shouldAnimate]];
    [persistCell setIntValue:[application isPersistent]];

    isFirstResponder = [autoLaunchValueField currentEditor] != nil;
    if (isFirstResponder == YES)
    {
        [autoLaunchValueField abortEditing];
        firstResponder = autoLaunchValueField;
    }
    [self setAutoLaunchDelay:[application autoLaunchDelay]];

    if (firstResponder != nil)
        [[self inspectorWindow] makeFirstResponder:firstResponder];
}

//----------------------------------------------------------------------

- (void) endEditing
{
    [super endEditing];
}

//----------------------------------------------------------------------

- (void) abortEditing
{
    [super abortEditing];

    [autoLaunchValueField abortEditing];
}

//----------------------------------------------------------------------

- (void) toggleOption:sender
{
    NSCell *selectedCell;
    int tag, state;
    FITile *tile;
    FIApplication *application;

    selectedCell = [sender selectedCell];
    tag = [selectedCell tag];
    state = [selectedCell state];
    
    tile = (FITile *)[self inspectedObject];
    application = [tile application];

    switch (tag)
    {
      case 0:
          [application setShouldAutoLaunch:state];
          [[application undoManager] setActionName:(state == YES) ? @"Set Auto Launch" : @"Clear Auto Launch"];
          break;

      case 1:
          [application setShouldAnimate:state];
          [[application undoManager] setActionName:(state == YES) ? @"Set Animate" : @"Clear Animate"];
          break;

      case 2:
          [[tile delegate] togglePersistenceOfTile:tile];
          //[[[application level] dock] togglePersistenceOfApplication:application];
          //[[application undoManager] setActionName:(state == YES) ? @"" : @""];
          break;

      default:
          break;
    }
}

//----------------------------------------------------------------------

- (void) willChangeInspectedObject
{
    [super willChangeInspectedObject];
    [[NSNotificationCenter defaultCenter] removeObserver:self name:FIApplicationDidChangeNotification object:nil];
    [[NSNotificationCenter defaultCenter] removeObserver:self name:FITileIconDidChangeNotification object:nil];
}

//----------------------------------------------------------------------

- (void) didChangeInspectedObject
{
    FIApplication *application;
    
    [super didChangeInspectedObject];

    application = [(FITile *)[self inspectedObject] application];
    if (application != nil)
    {
        NSNotificationCenter *defaultCenter;

        defaultCenter = [NSNotificationCenter defaultCenter];
        [defaultCenter addObserver:self
                       selector:@selector (applicationDidChange:)
                       name:FIApplicationDidChangeNotification
                       object:application];

        [defaultCenter addObserver:self
                       selector:@selector (applicationDidChange:)
                       name:FITileIconDidChangeNotification
                       object:[self inspectedObject]];
    }
}

//----------------------------------------------------------------------

- (void) applicationDidChange:(NSNotification *)aNotification
{
    [self updateInspector];
}

// XXX: What about showing/hiding the inspector?

//----------------------------------------------------------------------

- (TimeInterval) autoLaunchDelay
{
    TimeInterval timeInterval;

    timeInterval.timeValue = [autoLaunchValueField doubleValue];
    timeInterval.timeUnit = [autoLaunchUnitsPopUp indexOfSelectedItem];
    OBASSERT (timeInterval.timeUnit >= tuNone && timeInterval.timeUnit <= tuMinutes);
    if (timeInterval.timeUnit == tuNone)
        timeInterval.timeValue = 0;

    return timeInterval;
}

//----------------------------------------------------------------------

- (void) setAutoLaunchDelay:(TimeInterval)newDelay
{
    [autoLaunchValueField abortEditing];
    [autoLaunchValueField setDoubleValue:newDelay.timeValue];
    [autoLaunchUnitsPopUp selectItemAtIndex:newDelay.timeUnit - tuNone];
    [autoLaunchValueField setEnabled:(newDelay.timeUnit == tuNone) ? NO : YES];
}

//----------------------------------------------------------------------

- (void) changeDelayValue:sender
{
    TimeInterval newInterval;
    FIApplication *application;

    application = [(FITile *)[self inspectedObject] application];
    newInterval = [self autoLaunchDelay];
    //NSLog (@"newInterval: %@", NSStringFromTimeInterval (newInterval));
    [application setAutoLaunchDelay:newInterval];
    [[application undoManager] setActionName:@"Change Auto Launch Delay"];
}

//----------------------------------------------------------------------

- (void) changeDelayUnits:sender
{
    int index;
    TimeInterval newInterval;
    FIApplication *application;

    application = [(FITile *)[self inspectedObject] application];
    index = [autoLaunchUnitsPopUp indexOfSelectedItem];
    [autoLaunchValueField setEnabled:(index == 0) ? NO : YES];
    newInterval = [self autoLaunchDelay];
    [application setAutoLaunchDelay:newInterval];
    [[application undoManager] setActionName:@"Change Auto Launch Delay"];
    //NSLog (@"newInterval: %@", NSStringFromTimeInterval (newInterval));
}

@end
