#import "Fiend.h"

RCSID ("$Id$");

#import "SNInspector.h"

#import "SNInspectorManager.h"

@implementation SNInspector

// Nil window name -> no window -> use "No Inspector" inspector...  Got it?  Has this been inspected? I expected it to be!

- initWithWindowNibName:(NSString *)windowNibName
{
    if (windowNibName == nil)
    {
        // NSWindowController doesn't like a nil nib name.
        if ([super initWithWindow:nil] == nil)
            return nil;
    }
    else
    {
        if ([super initWithWindowNibName:windowNibName] == nil)
            return nil;
    }

    nonretainedInspectorManager = nil;

    inspectorView = nil;
    inspectedObject = nil;

    return self;
}

//----------------------------------------------------------------------

- (void) dealloc
{
    SNRelease (inspectorView);
    SNRelease (inspectedObject);

    [super dealloc];
}

//----------------------------------------------------------------------

- (SNInspectorManager *) inspectorManager
{
    return nonretainedInspectorManager;
}

//----------------------------------------------------------------------

- (void) setInspectorManager:(SNInspectorManager *)newInspectorManager
{
    nonretainedInspectorManager = newInspectorManager;
    // This shouldn't change after initialization, otherwise we'll have
    // to updated what is being inspected.
}

//----------------------------------------------------------------------

- inspectedObject
{
    return inspectedObject;
}

//----------------------------------------------------------------------

- (void) setInspectedObject:newObject
{
    if (newObject == inspectedObject)
        return;
    
    [self willChangeInspectedObject];
    SNRelease (inspectedObject);
    inspectedObject = [newObject retain];
    [self didChangeInspectedObject];
}

//----------------------------------------------------------------------

- (NSView *) inspectorView
{
    if ([self isWindowLoaded] == NO)
        [self loadInspector];

    return inspectorView;
}

//----------------------------------------------------------------------

- (NSWindow *) inspectorWindow
{
    //return [[self inspectorManager] window];
    return [inspectorView window]; // Or call method to trigger window load?
}

//----------------------------------------------------------------------

- (void) loadInspector
{
    // Forces window to be loaded if it isn't already.
    [self window];
}

//----------------------------------------------------------------------

// This makes it easier to ensure that it is only called once.

- (void) windowDidLoad
{
    [inspectorView retain];
}

//----------------------------------------------------------------------
// Subclass responsabilities
//----------------------------------------------------------------------

// Subclasses might check for a certain kind of class, or make sure the
// object conforms to a particular protocol...

- (BOOL) canInspectObject:anObject
{
    return NO;
}

//----------------------------------------------------------------------

- (void) updateInspector
{
    // Subclasses fill in inspector values based on inspected object.
}

//----------------------------------------------------------------------

// End editing and send actions to apply changes.

- (void) endEditing
{
    // This ends the editing of any text fields or text views.
    [[self inspectorWindow] endEditingFor:nil];
    // Subclasses may need to do more, for example, to deactivate NSColorWells.
}

//----------------------------------------------------------------------

// End editing, but don't send actions or apply changes.

- (void) abortEditing
{
}

//----------------------------------------------------------------------

- (void) willShowInspector
{
}

//----------------------------------------------------------------------

- (void) willHideInspector
{
    // Or have the inspector manager do this?  How is the message timing affected?
    // Want -willHideInspector, then setInspectedObject:nil (willChangeInspectedObject, didChangeInspectedObject)
    //[self setInspectedObject:nil];
    [self endEditing];
}

//----------------------------------------------------------------------

- (void) willChangeInspectedObject
{
    [self endEditing];
}

//----------------------------------------------------------------------

- (void) didChangeInspectedObject
{
    if ([self inspectedObject] != nil)
        [self updateInspector];
}

//----------------------------------------------------------------------
// Debugging
//----------------------------------------------------------------------

// XXX: What does it take to get this used?  *Must* descend from OBObject.

- (NSMutableDictionary *) debugDictionary
{
    NSMutableDictionary *debugDictionary;

    debugDictionary = [super debugDictionary];
    [debugDictionary setObject:inspectorView forKey:@"inspectorView"];
    [debugDictionary setObject:inspectedObject forKey:@"inspectedObject"];

    return debugDictionary;
}

@end
