#import "Fiend.h"

RCSID ("$Id$");

#import "SNInspectorManager.h"

#import "SNInspector.h"

// Would it make sense for this to know/enforce the inspectable class(es)?

@implementation SNInspectorManager

- initWithWindowNibName:(NSString *)windowNibName
{
    if ([super initWithWindowNibName:windowNibName] == nil)
        return nil;

    noInspectorView = nil;
    inspectorViewOrigin = NSZeroPoint;

    // Subclasses should set the initial mode in the init method.
    mode = nil;
    inspectorsByMode = [[NSMutableDictionary dictionary] retain];

    currentInspector = nil;
    inspectedObject = nil;

    return self;
}

//----------------------------------------------------------------------

- (void) dealloc
{
    SNRelease (noInspectorView);
    SNRelease (inspectorsByMode);
    SNRelease (currentInspector);
    SNRelease (inspectedObject);
    SNRelease (mode);

    [super dealloc];
}

//----------------------------------------------------------------------

- (void) windowDidLoad
{
    NSAssert (mode != nil, @"Expected mode to be set before window is loaded.");

    [noInspectorView retain];
    inspectorViewOrigin = [noInspectorView frame].origin;

    //if ([self isWindowLoaded] == YES)
    {
        //[currentInspector revert:self];
        [self updateCurrentInspector];
    }
}

//----------------------------------------------------------------------

- (void) addInspector:(SNInspector *)anInspector toMode:aMode
{
    NSMutableArray *modeInspectors;

    modeInspectors = [inspectorsByMode objectForKey:aMode];
    if (modeInspectors == nil)
    {
        modeInspectors = [NSMutableArray array];
        [inspectorsByMode setObject:modeInspectors forKey:aMode];
    }

    // Put it at the beginning, and then add inspectors from most general to most specific.
    [modeInspectors insertObject:anInspector atIndex:0];
    [anInspector setInspectorManager:self]; // XXX: Fix the types..
}

//----------------------------------------------------------------------

- (NSArray *) inspectorsForMode:aMode
{
    NSArray *inspectors;

    inspectors = [inspectorsByMode objectForKey:aMode];

    return inspectors;
}

//----------------------------------------------------------------------

- mode
{
    return mode;
}

//----------------------------------------------------------------------

- (void) setMode:newMode
{
    // If we use isEqual: instead, do we have to worry about nil modes?
    if (newMode == mode)
        return;

    SNRelease (mode);
    mode = [newMode retain];

    //if ([self isWindowLoaded] == YES)
    {
        // And then re-inspect...
        [self updateCurrentInspector];
    }
}

//----------------------------------------------------------------------

- inspectedObject
{
    return inspectedObject;
}

//----------------------------------------------------------------------

- (void) setInspectedObject:newObject
{
    if (newObject == inspectedObject)
        return;

    SNRelease (inspectedObject);
    inspectedObject = [newObject retain];

    //if ([self isWindowLoaded] == YES)
    {
        //[currentInspector revert:self];
        [self updateCurrentInspector];
    }
}

//----------------------------------------------------------------------

- (SNInspector *) currentInspector
{
    return currentInspector;
}

//----------------------------------------------------------------------

- (void) setCurrentInspector:(SNInspector *)newInspector
{
    NSView *inspectorView;
    
    if (currentInspector == nil)
    {
        [noInspectorView removeFromSuperview];
    }
    else
    {
        [currentInspector willHideInspector];
        [currentInspector setInspectedObject:nil];
        [[currentInspector inspectorView] removeFromSuperview];
    }

    SNRelease (currentInspector);
    currentInspector = [newInspector retain];

    // Where does it (the swapped in view) get placed?
    // The swapped views get placed at their current frame origin from the source window in the nib file...

    if ([currentInspector inspectorView] == nil)
    {
        inspectorView = noInspectorView;
    }
    else
    {
        inspectorView = [currentInspector inspectorView];
    }

    // Wait until after getting the inspectorView so that the window is guaranteed to be loaded.
    [currentInspector willShowInspector];
    [currentInspector setInspectedObject:inspectedObject];

    [inspectorView setFrameOrigin:inspectorViewOrigin];
    [[[self window] contentView] addSubview:inspectorView];

    // XXX: move this somewhere else?
    //[currentInspector revertChanges:self];
}

//----------------------------------------------------------------------

- (void) updateCurrentInspector
{
    NSArray *inspectors;
    NSEnumerator *inspectorEnumerator;
    SNInspector *inspector;
    
    if ([self isWindowLoaded] == NO)
        return;

    // 1. get inspected object
    // 2. take current mode
    // 3. get inspector for current mode
    inspectors = [self inspectorsForMode:mode];
    //NSLog (@"inspectors: %@", inspectors);

    // 4. set current inspector
    if (inspectors == nil)
    {
        [self setCurrentInspector:nil];
    }
    else
    {
        inspectorEnumerator = [inspectors objectEnumerator];
        while (inspector = [inspectorEnumerator nextObject])
        {
            //NSLog (@"trying %@", inspector);
            if ([inspector canInspectObject:inspectedObject] == YES)
            {
                break;
            }
        }

        //NSLog (@"using inspector: %@", inspector);

        // inspector may be nil if we reached the end of the list without finding a candidate.
        [self setCurrentInspector:inspector];
    }

    //NSLog (@"currentInspector: %@", currentInspector);
}

//----------------------------------------------------------------------
// NSWindow delegate
//----------------------------------------------------------------------

- (void) windowWillClose:(NSNotification *)aNotification
{
    // XXX: Updates defaults on whether to show inspector on start up.
    // Take frame autosave name?
}

//----------------------------------------------------------------------
// Other
//----------------------------------------------------------------------

- (NSMutableDictionary *) debugDictionary
{
    NSMutableDictionary *debugDictionary;

    debugDictionary = [super debugDictionary];
    [debugDictionary setObject:NSStringFromBool ([self isWindowLoaded]) forKey:@"isWindowLoaded"];
    [debugDictionary setObject:mode forKey:@"mode"];
    [debugDictionary setObject:inspectedObject forKey:@"inspectedObject"];
    [debugDictionary setObject:currentInspector forKey:@"currentInspector"];

    return debugDictionary;
}
#if 0
- (NSString *) description
{
    return [NSString stringWithFormat:@"<%@: nib loaded = %@, mode = %d, inspected object class = %@, current inspector = %@>",
                     NSStringFromClass ([self class]),
                     NSStringFromBool ([self isWindowLoaded]),
                     mode,
                     ((inspectedObject == nil)
                      ? @"<nil>"
                      : NSStringFromClass ([inspectedObject class])),
                     currentInspector];
}
#endif
@end
