#import "Fiend.h"

RCS_ID ("$Id$");

#import "FIApplication.h"

#import "FILevel.h"

DEFINE_NSSTRING (FIApplicationLocationDidChangeNotification);
DEFINE_NSSTRING (FIApplicationDidChangeNotification);

@implementation FIApplication

#define PL_ApplicationPath @"ApplicationPath"
#define PL_ShouldAnimate @"ShouldAnimate"
#define PL_ShouldAutoLaunch @"ShouldAutoLaunch"
#define PL_XTileOrigin @"XTileOrigin"
#define PL_YTileOrigin @"YTileOrigin"
#define PL_DelayTimeUnits @"DelayTimeUnits"
#define PL_DelayTimeValue @"DelayTimeValue"

- initWithRepresentedPath:(NSString *)aPath
{
    if ([super init] == nil)
        return nil;

    nonretainedLevel = nil;

    path = [aPath copy];
    location.x = 0;
    location.y = 0;

    shouldAutoLaunch = NO;
    shouldAnimate = NO;
    isPersistent = NO;

    autoLaunchDelay.timeValue = 0;
    autoLaunchDelay.timeUnit = tuNone;

    return self;
}

//----------------------------------------------------------------------

- (void) dealloc
{
    SNRelease (path);

    [super dealloc];
}

//======================================================================
//" NSCoding
//======================================================================

- (void) encodeWithCoder:(NSCoder *)aCoder
{
    [super encodeWithCoder:aCoder];

    //[aCoder encodeConditionalObject:nonretainedLevel];
    [aCoder encodeObject:path];
    [aCoder encodeValueOfObjCType:@encode (DockLocation) at:&location];
    [aCoder encodeValueOfObjCType:@encode (BOOL) at:&shouldAutoLaunch];
    [aCoder encodeValueOfObjCType:@encode (BOOL) at:&shouldAnimate];
    [aCoder encodeValueOfObjCType:@encode (BOOL) at:&isPersistent];
    [aCoder encodeValueOfObjCType:@encode (TimeInterval) at:&autoLaunchDelay];
}

//----------------------------------------------------------------------

- initWithCoder:(NSCoder *)aDecoder
{
    if ([super initWithCoder:aDecoder] == nil)
        return nil;

    nonretainedLevel = nil;
    //nonretainedLevel = [aDecoder decodeObject];
    path = [[aDecoder decodeObject] retain];
    [aDecoder decodeValueOfObjCType:@encode (DockLocation) at:&location];
    [aDecoder decodeValueOfObjCType:@encode (BOOL) at:&shouldAutoLaunch];
    [aDecoder decodeValueOfObjCType:@encode (BOOL) at:&shouldAnimate];
    [aDecoder decodeValueOfObjCType:@encode (BOOL) at:&isPersistent];
    [aDecoder decodeValueOfObjCType:@encode (TimeInterval) at:&autoLaunchDelay];

    return self;
}

//======================================================================
//" Loading and saving
//======================================================================

+ (FIApplication *) applicationFromPropertyList:(NSDictionary *)propertyList
{
    FIApplication *newApplication;
    id value;
    DockLocation aLocation;
    NSString *timeUnits;
    NSNumber *timeValue;

    newApplication = [[FIApplication alloc] initWithRepresentedPath:[propertyList objectForKey:PL_ApplicationPath]];

    value = [propertyList objectForKey:PL_ShouldAnimate];
    if (value != nil)
        [newApplication setShouldAnimate:SNBoolFromString (value)];

    value = [propertyList objectForKey:PL_ShouldAutoLaunch];
    if (value != nil)
        [newApplication setShouldAutoLaunch:SNBoolFromString (value)];

    aLocation.x = 0;
    aLocation.y = 0;

    value = [propertyList objectForKey:PL_XTileOrigin];
    if (value != nil)
        aLocation.x = [value intValue];
    
    value = [propertyList objectForKey:PL_YTileOrigin];
    if (value != nil)
        aLocation.y = [value intValue];

    [newApplication setLocation:aLocation];

    timeUnits = [propertyList objectForKey:PL_DelayTimeUnits];
    timeValue = [propertyList objectForKey:PL_DelayTimeValue];
    if (timeUnits != nil && timeValue != nil)
    {
        TimeInterval delay;
        delay.timeUnit = timeUnitFromString (timeUnits);
        delay.timeValue = [timeValue floatValue];
        [newApplication setAutoLaunchDelay:delay];
    }

    return [newApplication autorelease];
}

//----------------------------------------------------------------------

- (NSDictionary *) propertyListForApplication
{
    NSMutableDictionary *propertyList;

    propertyList = [NSMutableDictionary dictionary];
    [propertyList setObject:path forKey:PL_ApplicationPath];

    [propertyList setObject:NSStringFromBool (shouldAnimate) forKey:PL_ShouldAnimate];
    [propertyList setObject:NSStringFromBool (shouldAutoLaunch) forKey:PL_ShouldAutoLaunch];

    // Reading the property list doesn't accepts an unquoted negative value, which is what gets
    // used by setting the object to the NSNumber.
    [propertyList setObject:[NSString stringWithFormat:@"%d", location.x] forKey:PL_XTileOrigin];
    [propertyList setObject:[NSString stringWithFormat:@"%d", location.y] forKey:PL_YTileOrigin];

    [propertyList setObject:NSStringFromTimeUnit (autoLaunchDelay.timeUnit) forKey:PL_DelayTimeUnits];
    [propertyList setObject:[NSNumber numberWithFloat:autoLaunchDelay.timeValue] forKey:PL_DelayTimeValue];

    return propertyList;
}

//======================================================================
// Controlling general attributes
//======================================================================

- (NSString *) path
{
    return path;
}

//----------------------------------------------------------------------

- (NSString *) shortName
{
    return [[path lastPathComponent] stringByDeletingPathExtension];
}

//----------------------------------------------------------------------

- (FILevel *) level
{
    return nonretainedLevel;
}

//----------------------------------------------------------------------

- (void) setLevel:(FILevel *)newLevel
{
    // Use -[FILevel addApplication:]...
    nonretainedLevel = newLevel;
}

//----------------------------------------------------------------------

- (BOOL) shouldAutoLaunch
{
    return shouldAutoLaunch;
}

//----------------------------------------------------------------------

- (void) setShouldAutoLaunch:(BOOL)flag
{
    if (flag == shouldAutoLaunch)
        return;

    [[[self undoManager] prepareWithInvocationTarget:self] setShouldAutoLaunch:shouldAutoLaunch];
    shouldAutoLaunch = flag;
    [[NSNotificationCenter defaultCenter] postNotificationName:FIApplicationDidChangeNotification object:self];
}

//----------------------------------------------------------------------

- (BOOL) shouldAnimate
{
    return shouldAnimate;
}

//----------------------------------------------------------------------

- (void) setShouldAnimate:(BOOL)flag
{
    if (flag == shouldAnimate)
        return;

    [[[self undoManager] prepareWithInvocationTarget:self] setShouldAnimate:shouldAnimate];
    shouldAnimate = flag;
    [[NSNotificationCenter defaultCenter] postNotificationName:FIApplicationDidChangeNotification object:self];
}

//----------------------------------------------------------------------

- (BOOL) isPersistent
{
    return isPersistent;
}

//----------------------------------------------------------------------

- (void) setIsPersistent:(BOOL)flag
{
    if (flag == isPersistent)
        return;

    [[[self undoManager] prepareWithInvocationTarget:self] setIsPersistent:isPersistent];
    isPersistent = flag;
    [[NSNotificationCenter defaultCenter] postNotificationName:FIApplicationDidChangeNotification object:self];
}

//----------------------------------------------------------------------

- (TimeInterval) autoLaunchDelay
{
    return autoLaunchDelay;
}

//----------------------------------------------------------------------

- (void) setAutoLaunchDelay:(TimeInterval)newDelay
{
    if (newDelay.timeValue == autoLaunchDelay.timeValue && newDelay.timeUnit == autoLaunchDelay.timeUnit)
        return;

    [[[self undoManager] prepareWithInvocationTarget:self] setAutoLaunchDelay:autoLaunchDelay];
    autoLaunchDelay = newDelay;
    [[NSNotificationCenter defaultCenter] postNotificationName:FIApplicationDidChangeNotification object:self];
}

//======================================================================
// Controlling location
//======================================================================

- (DockLocation) location
{
    return location;
}

//----------------------------------------------------------------------

- (void) setLocation:(DockLocation)newLocation
{
    if (newLocation.x == location.x && newLocation.y == location.y)
        return;

    [[[self undoManager] prepareWithInvocationTarget:self] setLocation:location];
    location = newLocation;
    [[NSNotificationCenter defaultCenter] postNotificationName:FIApplicationLocationDidChangeNotification object:self];
}

//----------------------------------------------------------------------

- (void) rotateLocationClockwise:(BOOL)clockwise
{
    DockLocation newLocation;

    if (clockwise == YES)
    {
        newLocation.x = location.y;
        newLocation.y = -location.x;
    }
    else
    {
        newLocation.x = -location.y;
        newLocation.y = location.x;
    }

    [self setLocation:newLocation];
}

//======================================================================
// Debugging
//======================================================================

- (NSMutableDictionary *) debugDictionary
{
    NSMutableDictionary *debugDictionary;

    debugDictionary = [super debugDictionary];
    if (nonretainedLevel != nil)
        [debugDictionary setObject:[nonretainedLevel shortDescription] forKey:@"nonretainedLevel"];
    if (path != nil)
        [debugDictionary setObject:path forKey:@"path"];
    [debugDictionary setObject:NSStringFromDockLocation (location) forKey:@"location"];
    [debugDictionary setObject:NSStringFromBool (shouldAutoLaunch) forKey:@"shouldAutoLaunch"];
    [debugDictionary setObject:NSStringFromBool (shouldAnimate) forKey:@"shouldAnimate"];
    [debugDictionary setObject:NSStringFromBool (isPersistent) forKey:@"isPersistent"];

    return debugDictionary;
}

//======================================================================
// Other
//======================================================================

@end
