//
// $Id$
//

#import <Foundation/Foundation.h>
#import "FIModelComponent.h"

@class FIApplication;
@class FILevel;

extern NSString *FIDockDidAddLevelNotification;
extern NSString *FIDockDidRemoveLevelNotification;
extern NSString *FIDockDidMoveApplicationBetweenLevelsNotification;

@interface FIDock : FIModelComponent
{
    FILevel *persistentLevel;
    NSMutableArray *levels;
}

- init;
- (void) dealloc;

//======================================================================
//" Loading and saving
//======================================================================

- (void) loadDockFromPropertyList:(NSDictionary *)propertyList;
- (NSDictionary *) propertyListForDock;

//======================================================================
//" Controlling the document
//======================================================================

- (void) setDocument:(NSDocument *)newDocument;

//======================================================================
//" Controlling levels
//======================================================================

- (FILevel *) persistentLevel;
- (NSArray *) levels;

- (int) indexOfLevel:(FILevel *)level;
- (NSString *) displayNameOfLevel:(FILevel *)level;

- (void) addLevel:(FILevel *)level;
- (void) addLevel:(FILevel *)level atIndex:(int)index;
- (void) removeLevel:(FILevel *)level;
- (void) moveApplication:(FIApplication *)application toLevel:(FILevel *)newLevel;

//======================================================================
//" Debugging
//======================================================================

- (NSMutableDictionary *) debugDictionary;

//======================================================================
//" Other
//======================================================================

- (BOOL) containsApplicationWithPath:(NSString *)applicationPath;

- (void) removeApplicationsFromArray:(NSArray *)applications;

- (FIApplication *) findNextApplication:(FIApplication *)startingApplication
                             withString:(NSString *)searchString
                             ignoreCase:(BOOL)ignoreCase;

- (FIApplication *) findPreviousApplication:(FIApplication *)startingApplication
                                 withString:(NSString *)searchString
                                 ignoreCase:(BOOL)ignoreCase;

- (FILevel *) findNextLevel:(FILevel *)startingLevel withString:(NSString *)searchString ignoreCase:(BOOL)ignoreCase;
- (FILevel *) findPreviousLevel:(FILevel *)startingLevel withString:(NSString *)searchString ignoreCase:(BOOL)ignoreCase;

@end
