//
// $Id$
//

#import <AppKit/AppKit.h>
#import "FIModelComponent.h"

@class FIApplication;
@class FIDock;

extern NSString *FILevelNameDidChangeNotification;
extern NSString *FILevelDidAddApplicationNotification;
extern NSString *FILevelDidRemoveApplicationNotification;

@interface FILevel : FIModelComponent
{
    FIDock *nonretainedDock;

    NSString *levelName;
    NSMutableArray *applications;

    BOOL isPersistent;
}

- init;
- (void) dealloc;

//======================================================================
//" Loading and saving
//======================================================================

- (void) loadLevelFromPropertyList:(NSDictionary *)savedLevelPropertyList;
- (NSDictionary *) propertyListForLevel;

//======================================================================
//" Controlling the document
//======================================================================

- (void) setDocument:(NSDocument *)newDocument;

//======================================================================
//" Controlling general attributes
//======================================================================

- (FIDock *) dock;
- (void) setDock:(FIDock *)newDock;

- (NSString *) levelName;
- (void) setLevelName:(NSString *)newName;
- (NSString *) displayName;
- (NSString *) displayName:sender;

- (BOOL) isPersistent;
- (void) setIsPersistent:(BOOL)flag;

//======================================================================
//" Controlling applications
//======================================================================

- (NSArray *) applications;
- (BOOL) containsApplicationWithPath:(NSString *)applicationPath;

- (void) _addApplication:(FIApplication *)newApplication;
- (void) _removeApplication:(FIApplication *)anApplication;

- (void) addApplication:(FIApplication *)newApplication;
- (void) removeApplication:(FIApplication *)anApplication;

//======================================================================
//" Controlling locations
//======================================================================

- (BOOL) isDockLocationEmpty:(DockLocation)location;
- (void) rotateApplicationsClockwise:(BOOL)clockwise;

//======================================================================
//" Debugging
//======================================================================

- (NSMutableDictionary *) debugDictionary;

//======================================================================
//" Other
//======================================================================

- (void) clearLevel;

- (FIApplication *) findNextApplication:(FIApplication *)startingApplication
                             withString:(NSString *)searchString
                             ignoreCase:(BOOL)ignoreCase;

- (FIApplication *) findPreviousApplication:(FIApplication *)startingApplication
                                 withString:(NSString *)searchString
                                 ignoreCase:(BOOL)ignoreCase;

@end
