#import "Fiend.h"

RCS_ID ("$Id$");

#import "FIModelComponent.h"

@implementation FIModelComponent

//======================================================================
//" An FIModelComponent provides methods often used in the model
//" objects of a document based application that supports undo.  In
//" particular, it provides a reference back to the document, and gives
//" easy access to the documents undo manager.
//"
//" The document model objects are most often subclasses of
//" FIModelComponent.
//======================================================================

- init
{
    if ([super init] == nil)
        return nil;

    nonretainedDocument = nil;

    return self;
}

//----------------------------------------------------------------------

- (void) dealloc
{
    [[self undoManager] removeAllActionsWithTarget:self];
    [super dealloc];
}

//======================================================================
// NSCoding
//======================================================================

- (void) encodeWithCoder:(NSCoder *)aCoder
{
    [super encodeWithCoder:aCoder];

    [aCoder encodeConditionalObject:nonretainedDocument];
}

//----------------------------------------------------------------------

- initWithCoder:(NSCoder *)aDecoder
{
    if ([super initWithCoder:aDecoder] == nil)
        return nil;

    nonretainedDocument = [aDecoder decodeObject];

    return self;
}

//======================================================================
// Debugging
//======================================================================

- (NSMutableDictionary *) debugDictionary
{
    NSMutableDictionary *debugDictionary;

    debugDictionary = [super debugDictionary];
    if (nonretainedDocument != nil)
        [debugDictionary setObject:[nonretainedDocument shortDescription] forKey:@"nonretainedDocument"];

    return debugDictionary;
}

//======================================================================
// Other
//======================================================================

- document
{
    return nonretainedDocument;
}

//----------------------------------------------------------------------
//" Subclasses that contain other model components should override this
//" method to set the document for all of these components.
//----------------------------------------------------------------------

- (void) setDocument:(NSDocument *)newDocument
{
    if (newDocument == nonretainedDocument)
        return;

    OBPRECONDITION (nonretainedDocument == nil);
    OBPRECONDITION (newDocument != nil);

    // Since we remove all undo actions when we change the document, when we
    // remove a component from the document, we do *not* set the document to
    // nil.  If we did, the undo action would be removed and we wouldn't be
    // able to redo it.
    [[self undoManager] removeAllActionsWithTarget:self];
    nonretainedDocument = newDocument;
}

//======================================================================
// Provide easy access to the undo manager.
//======================================================================

- (NSUndoManager *) undoManager
{
    return [[self document] undoManager];
}

@end
