#import "Fiend.h"

RCS_ID ("$Id$");

#import "FITilePlacementView.h"

static NSImage *_blankTileImage = nil;
static NSImage *_fiendIconImage = nil;

// Assuming a non-flipped coordinate system...
void drawVerticalArrow (NSRect rect, NSSize arrowSize, BOOL pointingUp)
{
    PSgsave ();
    PSsetlinewidth (4);
    PSmoveto (NSMidX (rect), rect.origin.y);
    PSlineto (NSMidX (rect), NSMaxY (rect));
    if (pointingUp == YES)
    {
        PSmoveto (NSMidX (rect) - arrowSize.width / 2, NSMaxY (rect) - arrowSize.height);
        PSlineto (NSMidX (rect), NSMaxY (rect));
        PSlineto (NSMidX (rect) + arrowSize.width / 2, NSMaxY (rect) - arrowSize.height);
        PSstroke ();
    }
    else
    {
        PSmoveto (NSMidX (rect) - arrowSize.width / 2, rect.origin.y + arrowSize.height);
        PSlineto (NSMidX (rect), rect.origin.y);
        PSlineto (NSMidX (rect) + arrowSize.width / 2, rect.origin.y + arrowSize.height);
        PSstroke ();
    }
    PSgrestore ();
}

//----------------------------------------------------------------------

// Assuming a non-flipped coordinate system...
void drawHorizontalArrow (NSRect rect, NSSize arrowSize, BOOL pointingRight)
{
    PSgsave ();
    PSsetlinewidth (4);
    PSmoveto (rect.origin.x, NSMidY (rect));
    PSlineto (NSMaxX (rect), NSMidY (rect));
    if (pointingRight == YES)
    {
        PSmoveto (NSMaxX (rect) - arrowSize.width, NSMidY (rect) - arrowSize.height / 2);
        PSlineto (NSMaxX (rect), NSMidY (rect));
        PSlineto (NSMaxX (rect) - arrowSize.width, NSMidY (rect) + arrowSize.height / 2);
        PSstroke ();
    }
    else
    {
        PSmoveto (rect.origin.x + arrowSize.width, NSMidY (rect) - arrowSize.height / 2);
        PSlineto (rect.origin.x, NSMidY (rect));
        PSlineto (rect.origin.x + arrowSize.width, NSMidY (rect) + arrowSize.height / 2);
        PSstroke ();
    }
    PSgrestore ();
}

//======================================================================

@implementation FITilePlacementView

+ (void) initialize
{
    static BOOL initialized = NO;

    [super initialize];
    if (initialized == YES)
        return;
    initialized = YES;
    
    _blankTileImage = [[NSImage imageNamed:@"BlankTile" inBundleForClass:self] retain];
    _fiendIconImage = [[NSImage imageNamed:@"FiendIcon" inBundleForClass:self] retain];
}

//----------------------------------------------------------------------

- initWithFrame:(NSRect)frameRect
{
    if ([super initWithFrame:frameRect] == nil)
        return nil;
    
    delegate = nil;
    corner = SouthWestCorner;
    fillVertically = YES;

    numberTextFieldCell = [[NSTextFieldCell allocWithZone:[self zone]] initTextCell:@"1"];
    [numberTextFieldCell setAlignment:NSLeftTextAlignment];
    [numberTextFieldCell setDrawsBackground:NO];
    [numberTextFieldCell setTextColor:[NSColor blackColor]];
    [numberTextFieldCell setEditable:NO];
    [numberTextFieldCell setEnabled:YES];
    [numberTextFieldCell setWraps:NO];
    [numberTextFieldCell setFont:[NSFont controlContentFontOfSize:10]];

    return self;
}

//----------------------------------------------------------------------

- (void) dealloc
{
    // Right now, we just let IB set the delegate, so we don't try to release it.
    SNRelease (numberTextFieldCell);

    [super dealloc];
}

//----------------------------------------------------------------------

#define ARROW_HEIGHT_FACTOR (4)
#define ARROW_WIDTH_FACTOR (3)

#define NUMBER_HEIGHT (12)
#define NUMBER_WIDTH (12)

- (void) drawRect:(NSRect)rect
{
    NSRect boundsRect;
    float xMarker, yMarker;
    int flipFudge;

    flipFudge = ([self isFlipped] == YES) ? 0 : 1;

    // Now, wouldn't it be neat to get the user's background color or image to show in this view...
    boundsRect = [self bounds];
    [[NSColor controlBackgroundColor] set];
    NSRectFill (boundsRect);

    xMarker = boundsRect.size.width / 3;
    yMarker = boundsRect.size.height / 3;

    // fill the corner;
    {
        int x, y;

        x = (corner == 0 || corner == 1) ? 0 : 2;
        y = (corner == 0 || corner == 3) ? 0 : 2;

        // This part is assuming tiles of 64x64 -- no scaling is done to the images.  Therefore,
        // it will only work when the TilePlacementView is 192x192 (3*64,3*64)...
        // Perhaps this should always have it's bounds scaled to 192x192, and let the images scale too.

        if (_blankTileImage == nil)
        {
            [[NSColor darkGrayColor] set];
            PSrectfill (boundsRect.origin.x + x * xMarker, boundsRect.origin.y + y * yMarker,
                        xMarker, yMarker);
        }
        else
        {
            NSPoint targetPoint;

            targetPoint = NSMakePoint (boundsRect.origin.x + x * xMarker, boundsRect.origin.y + y * yMarker);
            [_blankTileImage compositeToPoint:targetPoint operation:NSCompositeSourceOver];

            if (_fiendIconImage != nil)
            {
                NSSize iconSize;

                iconSize = [_fiendIconImage size];
                targetPoint.x += (64 - iconSize.width) / 2;
                targetPoint.y += (64 - iconSize.height) / 2;

                [_fiendIconImage compositeToPoint:targetPoint operation:NSCompositeSourceOver];
            }
        }
    }

    [[NSColor blackColor] set];

    PSrectstroke (boundsRect.origin.x, boundsRect.origin.y + flipFudge, boundsRect.size.width - 1, boundsRect.size.height - 1);

    PSmoveto (boundsRect.origin.x + xMarker, boundsRect.origin.y);
    PSrlineto (0, boundsRect.size.height);
    PSmoveto (boundsRect.origin.x + 2 * xMarker, boundsRect.origin.y);
    PSrlineto (0, boundsRect.size.height);

    PSmoveto (boundsRect.origin.x, boundsRect.origin.y + yMarker);
    PSrlineto (boundsRect.size.width, 0);
    PSmoveto (boundsRect.origin.x, boundsRect.origin.y + 2 * yMarker);
    PSrlineto (boundsRect.size.width, 0);

    PSstroke ();

    {
        [[NSColor blackColor] set];

        if (fillVertically == YES)
        {
            BOOL goingUp;
            int l;
            int cornerColumn;

            NSRect arrowRect;

            goingUp = (corner == 0 || corner == 3);
            cornerColumn = (corner == 0 || corner == 1) ? 0 : 2;

            for (l = 0; l < 3; l++)
            {
                arrowRect.origin = boundsRect.origin;
                arrowRect.origin.x += l * xMarker;
                arrowRect.size = NSMakeSize (xMarker, boundsRect.size.height);
                if (l == cornerColumn)
                {
                    if (goingUp == YES)
                        arrowRect.origin.y += yMarker;
                    arrowRect.size.height -= yMarker;
                }
                arrowRect = NSInsetRect (arrowRect, 0, yMarker / 10);

                drawVerticalArrow (arrowRect, NSMakeSize (xMarker / ARROW_WIDTH_FACTOR, yMarker / ARROW_HEIGHT_FACTOR), goingUp);

                arrowRect.origin.x = NSMidX (arrowRect) + 4;
                arrowRect.origin.y = NSMidY (boundsRect) - NUMBER_HEIGHT / 2;
                arrowRect.size = NSMakeSize (NUMBER_WIDTH, NUMBER_HEIGHT);
                [numberTextFieldCell setIntValue:(cornerColumn == 0) ? (l + 1) : (3 - l)];
                [numberTextFieldCell drawWithFrame:arrowRect inView:self];
            }
        }
        else
        {
            BOOL goingRight;
            int l;
            int cornerRow;

            NSRect arrowRect;

            goingRight = (corner == 0 || corner == 1);
            cornerRow = (corner == 0 || corner == 3) ? 0 : 2;

            for (l = 0; l < 3; l++)
            {
                arrowRect.origin = boundsRect.origin;
                arrowRect.origin.y += l * yMarker;
                arrowRect.size = NSMakeSize (boundsRect.size.width, yMarker);
                if (l == cornerRow)
                {
                    if (goingRight == YES)
                        arrowRect.origin.x += xMarker;
                    arrowRect.size.width -= xMarker;
                }
                arrowRect = NSInsetRect (arrowRect, xMarker / 10, 0);

                drawHorizontalArrow (arrowRect, NSMakeSize (xMarker / ARROW_HEIGHT_FACTOR, yMarker / ARROW_WIDTH_FACTOR), goingRight);

                arrowRect.origin.x = NSMidX (boundsRect) - NUMBER_WIDTH / 2;
                arrowRect.origin.y = NSMidY (arrowRect) + 4;
                arrowRect.size = NSMakeSize (NUMBER_WIDTH, NUMBER_HEIGHT);
                [numberTextFieldCell setIntValue:(cornerRow == 0) ? (l + 1) : (3 - l)];
                [numberTextFieldCell drawWithFrame:arrowRect inView:self];
            }
        }
    }
}

//----------------------------------------------------------------------

- (void) mouseDown:(NSEvent *)theEvent
{
    NSRect boundsRect;
    float xMarker, yMarker;
    NSPoint hitPoint;

    boundsRect = [self bounds];

    xMarker = boundsRect.size.width / 3;
    yMarker = boundsRect.size.height / 3;

    hitPoint = [self convertPoint:[theEvent locationInWindow] fromView:nil];

    if (NSPointInRect (hitPoint, NSMakeRect (boundsRect.origin.x, boundsRect.origin.y,
                                             xMarker, yMarker)) == YES)
    {
        [self setCorner:SouthWestCorner];
        [delegate tilePlacementView:self didChangeTileCorner:corner];
    }
    else if (NSPointInRect (hitPoint, NSMakeRect (boundsRect.origin.x, NSMaxY (boundsRect) - yMarker,
                                                  xMarker, yMarker)) == YES)
    {
        [self setCorner:NorthWestCorner];
        [delegate tilePlacementView:self didChangeTileCorner:corner];
    }
    else if (NSPointInRect (hitPoint, NSMakeRect (NSMaxX (boundsRect) - xMarker, NSMaxY (boundsRect) - yMarker,
                                                  xMarker, yMarker)) == YES)
    {
        [self setCorner:NorthEastCorner];
        [delegate tilePlacementView:self didChangeTileCorner:corner];
    }
    else if (NSPointInRect (hitPoint, NSMakeRect (NSMaxX (boundsRect) - xMarker, boundsRect.origin.y,
                                                  xMarker, yMarker)) == YES)
    {
        [self setCorner:SouthEastCorner];
        [delegate tilePlacementView:self didChangeTileCorner:corner];
    }
    else
    {
        [self setFillVertically:![self fillVertically]];
        [delegate tilePlacementView:self didChangeVerticalPlacement:fillVertically];
    }
}

//----------------------------------------------------------------------

- (FiendTileCorner) corner
{
    return corner;
}

//----------------------------------------------------------------------

- (void) setCorner:(FiendTileCorner)newCorner
{
    if (newCorner < SouthWestCorner)
        newCorner = SouthWestCorner;
    if (newCorner > SouthEastCorner)
        newCorner = SouthEastCorner;
    corner = newCorner;
    [self setNeedsDisplay:YES];
}

//----------------------------------------------------------------------

- (BOOL) fillVertically
{
    return fillVertically;
}

//----------------------------------------------------------------------

- (void) setFillVertically:(BOOL)flag
{
    if (flag == fillVertically)
        return;

    fillVertically = flag;
    [self setNeedsDisplay:YES];
}

@end
